/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

define([
    "dojo/_base/declare",
    "dojo/promise/all",
    "aps/xhr",
    "dojo/_base/lang",
    "dojo/Deferred",
    "dojox/mvc/at",
    "dojox/mvc/getStateful",
    "dojox/mvc/getPlainValue",
    "aps/_View",
    "aps/Memory",
    "aps/Container",
    "aps/Output",
    "aps/load",
    "./../components/getUserResourcesGrid",
    "./../../messages",
    './../../checkboxes',
    "./../../common"
], function (
    declare,
    all,
    xhr,
    lang,
    Deferred,
    at,
    getStateful,
    getPlainValue,
    _View,
    Memory,
    Container,
    Output,
    load,
    getUserResourcesGrid,
    messages,
    CheckboxesHelper,
    common
) {
    return declare(_View, {
        _container: null,
        _tenant: null,
        _model: null,
        _users: null,
        _userQuota: null,
        _deltas: [],
        _checkboxNotificationsHelper: null,

        _submit: function () {
            var self = this,
                operation = {
                "call": {
                    "resourceId": self._tenant.aps.id,
                    "operation": "setUsersLicensesAfterUpsell",
                    "parameters": {
                        "data": {
                            provisionResource: self._users.map(function (user) {
                                return lang.mixin(getPlainValue(self._model), {user: {aps: {id: user.aps.id}}});
                            })
                        }
                    }
                }
            };

            return aps.biz.requestUsageChange({
                deltas: self._deltas,
                operations: [operation],
                subscriptionId: self._tenant.aps.subscription
            });
        },
        _renderForm: function () {
            var self = this;

            self._container.removeAll();
            self._container.set("title", _("Assign Cyber Protection Service for {count} user(s)").replace('{count}', self._users.length));
            if (self._userQuota) {
                self._container.addChild(new Output({
                    value: _("To assign service you need to define resource limits for the users you are now creating. You will be able to change them later.")
                }));
            }
            var form = ['aps/Tiles', {title: _('Service user settings')}, [
                ['aps/Tile',{title: _('Management Portal')},  [
                    ['aps/CheckBox', {
                        description: _('Quota overuse notification'),
                        checked: self._checkboxNotificationsHelper.getCheckboxAt('quota') || false
                    }],
                    ['aps/CheckBox', {
                        description: _('Schedule usage report'),
                        checked: self._checkboxNotificationsHelper.getCheckboxAt('reports') || false
                    }]
                ]],
				['aps/Tile', {title: _('Cyber Protection')}, [
					['aps/CheckBox', {
						description: _('Failure notifications'),
						checked: self._checkboxNotificationsHelper.getCheckboxAt('backup_error') || false
					}],
					['aps/CheckBox', {
						description: _('Warning notifications'),
						checked: self._checkboxNotificationsHelper.getCheckboxAt('backup_warning') || false
					}],
					['aps/CheckBox', {
						description: _('Success notifications'),
						checked: self._checkboxNotificationsHelper.getCheckboxAt('backup_info') || false
					}],
					['aps/CheckBox', {
						description: _('Daily recap about active alerts'),
						checked: self._checkboxNotificationsHelper.getCheckboxAt('backup_daily_report') || false
					}],
					['aps/CheckBox', {
						description: _('Malware detection notifications'),
						checked: self._checkboxNotificationsHelper.getCheckboxAt('anti_malware') || false
					}]
				]],
                ['aps/Tile', {title: _('Privileges')}, [
                    ['aps/FieldSet', [
                        ['aps/CheckBox', {
                            description: _('Administrator'),
                            checked: at(self._model, 'privileged').transform({
                                parse: function(value) {
                                    return value ? common.USER_PRIVILEGED.YES : common.USER_PRIVILEGED.NO;
                                },
                                format: function(value) {
                                    return value === common.USER_PRIVILEGED.YES;
                                }
                            })
                        }],
                        ['aps/Output', {description: _('An administrator can manage services of other users and back up Microsoft 365 seats.')}]
                    ]]
                ]]
            ]];

            load(form, self._container).then(function () {
                if (self._userQuota) {
                    getUserResourcesGrid(self._tenant, self._model, self._users.length).then(function (resources) {
                        self._container.addChild(resources.grid);
                        self._deltas = resources.deltas;
                        aps.apsc.hideLoading();
                    }, messages.displayError);
                }
                else {
                    aps.apsc.hideLoading();
                }
            }, messages.displayError);
        },
        init: function () {
            this._container = new Container();
            this._container.placeAt(this.domNode);
        },
        onHide: function () {
            messages.removeAll();
        },
        onPrev: function () {
            aps.apsc.prev();
        },
        onNext: function () {
            var self = this;
            if (!self._container.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }
            self._submit().then(function () {
                aps.apsc.next();
            });
        },
        onContext: function () {
            try {
                var self = this;
                var rql = false;

                if (aps.context.subscriptionId !== undefined) {
                    rql = "/aps/2/resources/?implementing(" + common.TYPES.TENANT + "),eq(aps.subscription," + aps.context.subscriptionId + "),select(profile)";
                } else if (aps.context.vars.tenant !== undefined) {
                    var id = aps.context.vars.tenant.aps.id;
                    rql = "/aps/2/resources/?implementing(" + common.TYPES.TENANT + "),eq(aps.id," + id + "),select(profile)";
                }

                if (!rql) {
                    throw new Error(_("Please check the account has a Backup Service."));
                }

                all({
                    resourcesToBind: aps.biz.getResourcesToBind(),
                    tenants: xhr.get(rql)
                }).then(function (response) {
                    self._users = response.resourcesToBind.users;
                    self._tenant = response.tenants.pop();

                    return all({
                        profile: common.getBackupProfile(self._tenant),
                        offeringItems: common.getBackupAccountResources(self._tenant)
                    });
                }).then(function (response) {
                    var profile = response.profile,
                        offeringItems = response.offeringItems;
                    self._userQuota = common.getUserQuota(profile);
                    self._model = getStateful({
                        aps: {
                            type: common.TYPES.USERS,
                            subscription: self._tenant.aps.subscription
                        },
                        tenant: {aps: {id: self._tenant.aps.id}},
                        notifications: profile.notifications || [],
                        privileged: common.USER_PRIVILEGED.NO,
                        activateDuringCreation: false,
                        userUsage: {}
                    });

                    self._checkboxNotificationsHelper = new CheckboxesHelper(self._model.notifications);
                    let offeringItemsNames = offeringItems.map(function (oi) {
                            return window.offeringItemsMap[oi.property];
                        }),
                        limitsValue = self._userQuota ? 0 : common.QUOTA_UNLIMITED_VALUE;

                    self._model.set("offeringItems", offeringItemsNames.reduce(function (offeringItems, offeringItemName) {
                        offeringItems.push(getStateful({limit: limitsValue, name: offeringItemName}));
                        return offeringItems;
                    }, []));

                    self._renderForm();
                }, messages.displayError);

            } catch (e) {
                console.log(e);
                messages.displayError(e);
            }
        }
    });
});



