/*
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

define([
    'dojo/_base/declare',
    'dojo/promise/all',
    'dijit/registry',
    'aps/_View',
    'aps/xhr',
    'aps/Output',
    'aps/Tile',
    'aps/nav/ViewPluginManager',
    './../components/getTenantButtons',
    './../components/getAdminGrid',
    './../../common',
    './../../tags',
    './../../messages',
    '../../wizardData'
], function (
    declare,
    all,
    registry,
    _View,
    xhr,
    Output,
    Tile,
    ViewPluginManager,
    getTenantButtons,
    getAdminGrid,
    common,
    tags,
    messages,
    wizardData
) {
    return declare(_View, {
        _administrationTilesId: null,
        _topTilesId: null,
        _administratorsContainerId: null,
        _tenant: null,
        _viewId: 'nextcp-ccp-partner',

        _getSubscriptionMediator: function () {
            var managerSubscription = new ViewPluginManager('subscriptionPlugin');
            return managerSubscription.init().then(function () {
                return managerSubscription.get('mediator');
            });
        },

        _getGettingStartedTile: function (self, additionalButtons, administratorGuideLink) {
            var tile = new Tile({
                    title: _('Getting started'),
                    gridSize: 'md-8 xs-12',
                    buttons: [
                        {
                            iconClass: 'fa fa-external-link',
                            title: _('Log into Management Console'),
                            autoBusy: false,
                            onClick: function () {
                                var w = window.open('', '_blank');
                                xhr.get('/aps/2/resources/' + self._tenant.aps.id + '/redirectUrl').then(function (response) {
                                    w.location.href = response.url;
                                }, function (error) {
                                    w.close();
                                    console.error('ERR: \n' + JSON.stringify(error));
                                    messages.displayError(error);
                                    self.cancel();
                                });
                            }
                        }
                    ].concat(additionalButtons)
                }),
                gettingStartedOutput = new Output({
                    value: _('This is a unique integration of backup with full-stack next-generation anti-malware protection and comprehensive endpoint management tools. This synergy eliminates complexity, so service providers can protect customers better while keeping costs down.') +
                        tags.br() + _('Log into Management Console as an administrator to manage your customers and their backups.') +
                        tags.br() + _('Learn more about the service - Administrator\'s guide.') +
                        tags.br() + tags.a(administratorGuideLink, _('Administrator\'s guide'), {'target': '_blank'}),
                    escapeHTML: false
                });

            tile.addChild(gettingStartedOutput);

            return tile;
        },
        showMessages: function () {
            var self = this,
                data = wizardData.getWizardData(self._viewId);
            if (data && data.message) {
                messages.displayMessage(data.message.text, data.message.type);
                wizardData.resetWizardData(self._viewId);
            }
        },
        init: function () {
            var self = this;

            self._subscriptionTilesId = self.genId('subscription_tiles');
            self._administratorsContainerId = self.genId('administrators_container');

            return [
                ['aps/Tiles', {
                    id: self._subscriptionTilesId
                }, []],
                ['aps/Container', {
                    id: self._administratorsContainerId,
                    title: _('Administrators'),
                    gridSize: 'md-12 xs-12'
                }]
            ];
        },
        onHide: function () {
            messages.removeAll();
        },
        onContext: function () {
            var self = this,
                tenant = aps.context.vars.tenant;
            self._tenant = tenant;

            /**
             * Statistics Widget
             *
             * @param response
             */
            var drawSubscriptionTiles = function (response) {
                var subscriptionId = tenant.subscriptionId,
                    subscriptionTiles = self.byId(self._subscriptionTilesId),
                    buttons = getTenantButtons(response.backupStatistic.tenant, subscriptionId, tenant.subscription.trial, false);

                subscriptionTiles.removeAll();
                subscriptionTiles.addChild(self._getGettingStartedTile(self, buttons, response.administratorGuideLink));
            };

            all({
                backupStatistic: xhr.get('/aps/2/resources/' + tenant.aps.id + '/updateStatistics'),
                administratorGuideLink: xhr.get('/aps/2/resources/' + tenant.aps.id + '/administratorGuideLink'),
                backupAdminGrid: getAdminGrid(self._tenant)
            }).then(function (response) {
                var backupAdminGrid = response.backupAdminGrid,
                    adminContainer = self.byId(self._administratorsContainerId);
                adminContainer.removeAll();
                adminContainer.addChild(backupAdminGrid);

                drawSubscriptionTiles(response);
                self.showMessages();
                aps.apsc.hideLoading();
            }, messages.displayError);
        }
    });
});

