/*
 * @Copyright © 2002-2022 Acronis International GmbH. All rights reserved
 */

define([
    'dojo/_base/declare',
    'aps/_View',
    './../components/getAdminGrid',
    './../../messages',
    '../../wizardData',
    '../../common',
    'aps/ResourceStore',
    'aps/Container',
    'aps/Output',
    'aps/Grid',
    'dojo/promise/all',
    'dijit/registry',
    'aps/Memory',
    'aps/Button'
], function (
    declare,
    _View,
    getAdminGrid,
    messages,
    wizardData,
    common,
    Store,
    Container,
    Output,
    Grid,
    all,
    registry,
    Memory,
    Button
) {
    return declare(_View, {
        _administratorsContainerId: null,
        _servicePlansContainerId: null,
        _tenant: null,
        _tabContainerId: null,
        _currentTabContainerId: null,
        _viewId: 'nextcp-ccp-reseller',
        init: function () {
            var self = this;
            self._tabContainerId = self.genId('tab_container');
            self._servicePlansContainerId = self.genId('service_plans');
            self._administratorsContainerId = self.genId('administrators_container');
            self._currentTabContainerId = self._administratorsContainerId;
            return ["aps/Tabs", {
                id: self._tabContainerId,
                activeTabId: self._administratorsContainerId,
                onClick: function () {
                    var tabContainer = self.byId(self._tabContainerId);
                    if (tabContainer.activeTabId !== self._currentTabContainerId) {
                        self._currentTabContainerId = tabContainer.activeTabId;
                        aps.apsc.showLoading();
                        switch (self._currentTabContainerId) {
                            case self._administratorsContainerId:
                                self.initializeAdministratorsTab();
                                break;
                            case self._servicePlansContainerId:
                                self.initializeServicePlansTab();
                                break;
                        }
                    }
                }
            },
                [
                    ['aps/Tab',
                        {
                            id: self._administratorsContainerId,
                            tabTitle: _('Administrators')
                        }
                    ],
                    [
                        'aps/Tab',
                        {
                            id: self._servicePlansContainerId,
                            tabTitle: _('Cloud Storage')
                        }
                    ]
                ]
            ];
        },
        onHide: function () {
            messages.removeAll();
        },
        onContext: function () {
            var self = this;
            self._tenant = aps.context.vars.tenant;
            switch (self._currentTabContainerId) {
                case self._administratorsContainerId:
                    self.initializeAdministratorsTab();
                    break;
                case self._servicePlansContainerId:
                    self.initializeServicePlansTab();
                    break;
            }
            self.showMessages();
        },
        showMessages: function () {
            var self = this,
                data = wizardData.getWizardData(self._viewId);
            if (data && data.message) {
                messages.displayMessage(data.message.text, data.message.type);
                wizardData.resetWizardData(self._viewId);
            }
        },
        initializeAdministratorsTab: function () {
            var self = this;
            var params = {
                newUserUrl: 'http://www.acronis.com/aps/cloud-backup-v1#nextcp-ccp-reseller-administrator-new',
                editUserUrl: 'http://www.acronis.com/aps/cloud-backup-v1#nextcp-ccp-reseller-administrator-edit'
            };
            getAdminGrid(self._tenant, params).then(function (backupAdminGrid) {
                var adminContainer = self.byId(self._administratorsContainerId);
                adminContainer.removeAll();
                adminContainer.addChild(backupAdminGrid);
                self._administratorsTabInitialized = true;
                aps.apsc.hideLoading();
            }, messages.displayError);
        },
        initializeServicePlansTab: function () {
            var self = this;
            var tenantId = aps.context.vars.tenant.aps.id,
                storages,
                servicePlansStore = new Store({
                    target: "/aps/2/resources/" + tenantId + "/plans"
                }),
                storagesStore = new Store({
                    target: "/aps/2/resources/" + tenantId + "/storages"
                }),
                columns = [
                    {
                        field: "name",
                        name: _("Service Plan name"),
                        sortable: false
                    },
                    {
                        field: "planId",
                        name: _("ID"),
                        sortable: false
                    },
                    {
                        field: "storageId",
                        name: _("Storage"),
                        sortable: false,
                        escapeHTML: false,
                        renderCell: function (row, data) {
                            if (row.storagePolicy.toString() === common.STORAGE_POLICY.PARENT) {
                                return _("Provider's storage");
                            }
                            if (data) {
                                return common.escapeAngularBrackets(_("__storageName__ (__datacenterName__)", {
                                    storageName: row.storageName,
                                    datacenterName: row.datacenterName
                                }));
                            }

                            return '<img src="images/statuses/warning24.png">' + " " + _("Not assigned");
                        }
                    },
                    {
                        field: "storageId",
                        name: _(""),
                        escapeHTML: false,
                        sortable: false,
                        align: "right",
                        renderCell: function (row, data) {
                            if (row.storagePolicy.toString() === common.STORAGE_POLICY.PARENT) {
                                return "";
                            }
                            return new Button({
                                iconClass: "sb-modify-selected",
                                label: data ? _("Edit") : _("Assign"),
                                onClick: function () {
                                    if (storages.length) {
                                        wizardData.setWizardData(row, 'nextcp-ccp-reseller-assign-storage');
                                        aps.apsc.gotoView('nextcp-ccp-reseller-assign-storage');
                                    } else {
                                        messages.displayWarning(_("There is no storage to assign to the service plan."));
                                        this.cancel();
                                    }
                                }
                            });
                        }
                    }
                ];

            var servicePlansTabContainer = self.byId(self._servicePlansContainerId);
            servicePlansTabContainer.removeAll();

            var informationContainer = new Container({
                id: 'informationContainer',
                title: _('Information'),
                visible: false

            });
            informationContainer.addChild(new Output({
                content: _("You may use your own storage to keep the customers' data there.")
            }));
            informationContainer.addChild(new Output({
                content: _("The storage can be registered in the management console.")
            }));
            servicePlansTabContainer.addChild(informationContainer);

            var servicePlanContainer = new Container({
                id: 'servicePlansContainer',
                title: _('Service Plans'),
                visible: false
            });
            servicePlanContainer.addChild(new Output({
                id: 'notAssignedWarning',
                content: '<img src="images/statuses/warning24.png">' + _('Subscription provisioning will fail for Service Plans without storage assigned.'),
                visible: false
            }));
            servicePlanContainer.addChild(new Grid({
                id: 'servicePlansGrid',
                pageSizeOptions: 'GRID_UNLIMITED',
                showPaging: false,
                columns: columns
            }));
            servicePlansTabContainer.addChild(servicePlanContainer);

            all({
                plans: servicePlansStore.query(),
                storages: storagesStore.query()
            }).then(function (result) {
                var plans = result.plans,
                    notAssignedExist = plans.some(function (plan) {
                        return plan.storagePolicy.toString() === common.STORAGE_POLICY.OWN && !plan.storageId;
                    });

                storages = result.storages;
                registry.byId('informationContainer').set('visible', true);
                registry.byId('servicePlansContainer').set('visible', true);
                registry.byId("servicePlansGrid").set("store", new Memory({data: plans, idProperty: "planId"}));
                registry.byId("notAssignedWarning").set("visible", notAssignedExist);
                aps.apsc.hideLoading();

            }, messages.displayError);

        }
    });
});

