/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
    "aps/Memory",
    "aps/_View",
    "aps/xhr",
    "dojo/_base/declare",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getPlainValue",
    "dojox/mvc/getStateful",
    "./../../../messages",
    "./../../../wizardData",
    "aps/ResourceStore"
], function (
    Memory,
    _View,
    xhr,
    declare,
    registry,
    at,
    getPlainValue,
    getStateful,
    messages,
    wizardData,
    Store
) {

    return declare(_View, {
        _tenantId: null,
        _servicePlan: null,
        _storageModel: null,
        init: function () {
            return [
                ["aps/Panel", [
                    ["aps/FieldSet", {title: _("Assign storage")}, [
                        ["aps/Output", {
                            id: 'servicePlanName',
                            gridSize: 'md-6 xs-12',
                            label: _("Service Plan name")
                        }],
                        ["aps/Select", {
                            id: "storage",
                            gridSize: 'md-6 xs-12',
                            label: _("Select storage"),
                            description: _("Changing the storage for the plan will not impact the existing subscriptions.")
                        }]
                    ]]
                ]]
            ];
        },
        onHide: function () {
            messages.removeAll();
        },
        onContext: function (context) {

            var self = this;
            self._tenantId = context.vars.tenant.aps.id;

            var storagesStore = new Store({
                target: "/aps/2/resources/" + self._tenantId + "/storages"
            });

            storagesStore.query().then(function (storages) {

                self._servicePlan = wizardData.getWizardData('nextcp-ccp-reseller-assign-storage');
                self._storageModel = getStateful({parameters: {}});

                var storagesOptions = storages.map(function (storage) {
                    return {
                        id: storage.datacenterId + '_' + storage.id,
                        label: _("__storageName__ (__datacenterName__)", {
                            storageName: storage.name,
                            datacenterName: storage.datacenterName
                        })
                    };
                });

                if (self._servicePlan.datacenterId && self._servicePlan.storageId) {
                    self._storageModel.set("parameters", {
                        datacenterId: self._servicePlan.datacenterId,
                        storageId: self._servicePlan.storageId
                    });
                } else if (storages.length) {
                    self._storageModel.set("parameters", {
                        datacenterId: storages[0].datacenterId,
                        storageId: storages[0].id
                    });
                }

                registry.byId('servicePlanName').set('value', self._servicePlan.name);
                registry.byId('storage').set('value', at(self._storageModel, "parameters").transform({
                    format: function (value) {
                        if (value.datacenterId && value.storageId) {
                            return value.datacenterId + '_' + value.storageId;
                        }
                    },
                    parse: function (value) {
                        var parts = value.split('_');
                        return {
                            datacenterId: parts[0],
                            storageId: parts[1]
                        };
                    }
                }));
                registry.byId('storage').set('store', new Memory({data: storagesOptions, idProperty: 'id'}));
                aps.apsc.hideLoading();

            }, messages.displayError);
        },
        onSubmit: function () {
            var self = this;
            xhr.put("/aps/2/resources/" + self._tenantId + "/plans/" + self._servicePlan.planId, {
                data: JSON.stringify(getPlainValue(self._storageModel.parameters)),
                headers: {"Content-Type": "application/json"}
            }).then(function () {
                aps.apsc.gotoView('nextcp-ccp-reseller');
            }, messages.displayError);
        }
    });
});
