/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
    "aps/Memory",
    "aps/_View",
    "aps/xhr",
    "aps/FieldSet",
    "dojo/_base/declare",
    "dijit/registry",
    "dojox/mvc/at",
    "dojox/mvc/getPlainValue",
    "dojox/mvc/getStateful",
    "./../../../common",
    "./../../../messages",
    "./../../../wizardData"
], function (
    Memory,
    _View,
    xhr,
    FieldSet,
    declare,
    registry,
    at,
    getPlainValue,
    getStateful,
    common,
    messages,
    wizardData
) {

    return declare(_View, {
        _admin: null,
        _adminModel: null,
        _tenant: null,
        _administratorId: null,
        _initialize: function () {
            var self = this;
            return xhr.get("/aps/2/resources/" + self._tenant.aps.id + "/administrators/" + self._administratorId, {
                query: {
                    datacenterId: self._datacenterId
                }
            }).then(function (response) {
                self._admin = response;
                self._adminModel.set({
                    login: response.login,
                    email: response.email,
                    firstName: response.firstName,
                    lastName: response.lastName,
                    notifications: common.transformNotificationsToObject(response.notifications)
                });
                aps.apsc.hideLoading();
            }, messages.displayError);
        },
        init: function () {
            var self = this;
            self._adminModel = getStateful({
                notifications: {}
            });
            return [
                ["aps/Panel", [
                    ["aps/FieldSet", {title: _("General information")}, [
                        ["aps/FieldSet", [
                            ["aps/Output", {
                                id: self.genId("login"),
                                label: _("Login"),
                                gridSize: "md-6 xs-12",
                                value: at(self._adminModel, "login")
                            }],
                            ["aps/TextBox", {
                                id: self.genId("email"),
                                label: _("Email"),
                                gridSize: "md-6 xs-12",
                                value: at(self._adminModel, "email"),
                                required: true,
                                trim: true,
                                pattern: '^[^\\s@]+@[^\\s@]+\\.[^\\s@.]+$',
                                invalidMessage: _('Please enter a valid email.')
                            }]
                        ]]
                    ]],
                    ["aps/FieldSet", {title: _("Optional")}, [
                        ["aps/FieldSet", [
                            ["aps/TextBox", {
                                id: self.genId("first_name"),
                                label: _("First name"),
                                required: false,
                                optionalText: "",
                                trim: true,
                                gridSize: "md-6 xs-12",
                                value: at(self._adminModel, "firstName")
                            }],
                            ["aps/TextBox", {
                                id: self.genId("last_name"),
                                label: _("Last name"),
                                required: false,
                                optionalText: "",
                                trim: true,
                                gridSize: "md-6 xs-12",
                                value: at(self._adminModel, "lastName")
                            }]
                        ]]
                    ]],
                    ["aps/FieldSet", {"id": common.genId('fieldset'), title: _("Notifications")}, [
                        ["aps/FieldSet", {
                            label: _('MANAGEMENT PORTAL')
                        }, [
                            ["aps/CheckBox", {
                                label: _('Quota overuse notifications'),
                                checked: at(self._adminModel.notifications, "quota"),
                                gridSize: 'md-4'
                            }],
                            ["aps/CheckBox", {
                                label: _('Scheduled usage reports'),
                                checked: at(self._adminModel.notifications, "reports"),
                                gridSize: 'md-4'
                            }]
                        ]],
                        ["aps/FieldSet", {
                            label: _('CYBER PROTECTION')
                        }, [
                            ["aps/CheckBox", {
                                label: _('Failure notifications'),
                                checked: at(self._adminModel.notifications, "backup_error"),
                                gridSize: 'md-4'
                            }],
                            ["aps/CheckBox", {
                                label: _('Warning notifications'),
                                checked: at(self._adminModel.notifications, "backup_warning"),
                                gridSize: 'md-4'
                            }],
                            ["aps/CheckBox", {
                                label: _('Success notifications'),
                                checked: at(self._adminModel.notifications, "backup_info"),
                                gridSize: 'md-4'
                            }],
                            ["aps/CheckBox", {
                                label: _('Daily recap about active alerts'),
                                checked: at(self._adminModel.notifications, "backup_daily_report"),
                                gridSize: 'md-4'
                            }],
                            ["aps/CheckBox", {
                                label: _('Malware detection notifications'),
                                checked: at(self._adminModel.notifications, "anti_malware"),
                                gridSize: 'md-4'
                            }]
                        ]]
                    ]]
                ]]
            ];
        },
        onHide: function () {
            messages.removeAll();
        },
        onContext: function (context) {
            var self = this;
            self._tenant = context.vars.tenant;
            self._administratorId = aps.context.params.id;
            self._datacenterId = aps.context.params.datacenterId;
            self._initialize();
        },
        onSubmit: function () {
            var self = this,
                tenant = self._tenant,
                admin = self._admin;

            if (!self.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }

            var data = getPlainValue(self._adminModel);
            if (Object.keys(data.notifications).length) {
                data.notifications = common.transformNotificationsToArray(data.notifications);
            } else {
                delete data.notifications;
            }

            xhr("/aps/2/resources/" + tenant.aps.id + "/administrators/" + self._administratorId, {
                method: "PUT",
                query: {
                    datacenterId: self._datacenterId
                },
                data: JSON.stringify(data),
                headers: {"Content-Type": "application/json"}
            }).then(function () {
                if (admin.status != common.STATUS.INACTIVE && admin.email !== data.email) {
                    var message = {
                        text: _("The confirmation message has been sent to email '{0}'.").replace("{0}", data.email),
                        type: "info"
                    };
                    wizardData.setWizardData({message: message}, 'nextcp-ccp-reseller');
                }
                aps.apsc.gotoView("nextcp-ccp-reseller");
            }, messages.displayError);
        }
    });
});
