/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "aps/load",
    "dojo/Deferred",
    "app/common",
    "app/messages",
    "app/storagesHelper",
    "dojo/promise/all"
], function (
    load,
    Deferred,
    common,
    messages,
    storagesHelper,
    all
) {
    var globals = aps.context.vars.globals,
        profile = aps.context.vars.profile,
        storageLocation = profile.storageLocation,
        isFreemiumMailboxStorageEnabled = globals.minimalCommitment; // Enabled only for Ingram

    function fetchStorageOrDatacenterOutput() {
        var deferred = new Deferred();
        storagesHelper.loadDatacenters(globals).then(function (allDatacenters) {
            if (profile.mspGroupType == common.GROUP_TYPE.PARTNER) {
                var profileDatacenters = storagesHelper.filterDatacenters(allDatacenters, profile);
                var datacentersOutput = storagesHelper.makeDatacentersOutput(profileDatacenters);
                deferred.resolve(datacentersOutput);
                return;
            }
            storagesHelper.loadStorages(allDatacenters).then(function (allStorages) {
                var profileStorages = storagesHelper.filterStorages(allStorages, profile),
                    storagesOutput = storagesHelper.makeStoragesOutput(profileStorages, profile);
                deferred.resolve(storagesOutput);
            }, deferred.reject);
        }, deferred.reject);
        return deferred;
    }

    function fetchFcStorageOutput(){
        var deferred = new Deferred();
        storagesHelper.loadDatacenters(globals).then(function (allDatacenters) {
            storagesHelper.loadInfras(allDatacenters).then(function (infras){
               var fcStorageOutput = storagesHelper.makeFcStorageOutput(infras, profile);
               deferred.resolve(fcStorageOutput);

            });
        }, deferred.reject);
        return deferred;
    }

    all({
        storageOrDatacenterOutput: fetchStorageOrDatacenterOutput(),
        fcStorageOutput: fetchFcStorageOutput()
    }).then(function ({storageOrDatacenterOutput, fcStorageOutput}) {
        var widgets = ["aps/PageContainer", {id: "apsPageContainer"}, [
            ["aps/FieldSet", {id: common.genId('fieldset'), title: _("General")}, [
                ["aps/Output", {
                    id: common.genId("name"),
                    label: _("Profile name"),
                    value: profile.name
                }],
                ["aps/Output", {
                    id: common.genId("resourceType"),
                    label: _("Resource"),
                    value: profile.resourceTypeId
                }]
            ]],
            ["aps/FieldSet", {id: common.genId('fieldset'), title: _("Options")}, [
                ["aps/Output", {
                    id: common.genId('fieldset'),
                    label: _("Account type"),
                    value: profile.mspGroupType == common.GROUP_TYPE.PARTNER ? _("Partner") : _("Customer")
                }],
                ["aps/Output", {
                    id: common.genId("output"),
                    label: profile.mspGroupType == common.GROUP_TYPE.PARTNER ? _("Datacenter") : _("Storage"),
                    value: storageOrDatacenterOutput,
                    escapeHTML: false
                }],
                ["aps/Output", {
                    id: common.genId('fcStorage'),
                    label: _("Files Cloud Storage"),
                    value: fcStorageOutput,
                    visible: profile.mspGroupType == common.GROUP_TYPE.CUSTOMER
                }],
                ["aps/Output", {
                    id: common.genId("storageLocation"),
                    label: _("Backup locations"),
                    value: storageLocation == common.BACKUP_LOCATIONS.LOCAL_AND_CLOUD ? _("Local and cloud backups") : _("Cloud only backups"),
                    visible: profile.storageLocation != null
                }],
                ["aps/Output", {
                    id: common.genId('output'),
                    visible: profile.mspGroupType != common.GROUP_TYPE.PARTNER,
                    label: _("Manage user quotas"),
                    value: profile.userQuota != common.USER_QUOTA.OFF ? _("Yes") : _("No")
                }],
                ["aps/Output", {
                    id: common.genId('output'),
                    visible: profile.mspGroupType != common.GROUP_TYPE.PARTNER,
                    label: _("Do not allow to set user quotas exceeding subscription limits"),
                    value: profile.userQuota == common.USER_QUOTA.STRICT ? _("Yes") : _("No")
                }],
                ["aps/Output", {
                    id: common.genId('output'),
                    label: _("Free cloud storage per Microsoft 365 seat (GB)"),
                    visible: isFreemiumMailboxStorageEnabled && profile.mspGroupType != common.GROUP_TYPE.PARTNER,
                    value: profile.freemiumMailboxStorage == common.QUOTA_UNLIMITED_VALUE ?
                        _("Unlimited") :
                        Math.round(Number(profile.freemiumMailboxStorage || 0) / common.BYTES_IN_GB)
                }]
            ]],
            ["aps/FieldSet", {title: _("Notifications")}, [
                ["aps/FieldSet", {
                    label: _('MANAGEMENT PORTAL'),
                    class: 'acronis-notifications-section-title',
                    title: false
                }],
                ["aps/Output", {
                    label: _('Quota overuse notifications'),
                    value: common.printNotification(profile.notifications, 'quota')
                }],
                ["aps/Output", {
                    label: _('Scheduled usage reports'),
                    value: common.printNotification(profile.notifications, 'reports')
                }],
                ["aps/Output", {
                    class: 'space-before-element'
                }],
                ["aps/FieldSet", {
                    label: _('CYBER PROTECTION'),
                    class: 'acronis-notifications-section-title',
                    title: false
                }],
                ["aps/Output", {
                    label: _('Failure notifications'),
                    value: common.printNotification(profile.notifications, 'backup_error')
                }],
                ["aps/Output", {
                    label: _('Warning notifications'),
                    value: common.printNotification(profile.notifications, 'backup_warning')
                }],
                ["aps/Output", {
                    label: _('Success notifications'),
                    value: common.printNotification(profile.notifications, 'backup_info')
                }],
                ["aps/Output", {
                    label: _('Daily recap about active alerts'),
                    value: common.printNotification(profile.notifications, 'backup_daily_report')
                }],
				["aps/Output", {
					label: _('Malware detection notifications'),
					value: common.printNotification(profile.notifications, 'anti_malware')
				}]
            ]]
        ]];

        load(widgets).then(function () {
            aps.app.onSubmit = function () {
                aps.apsc.gotoView(common.getRelatedViewId('edit'), profile.aps.id);
            };
        }, messages.displayError);
    }, messages.displayError);

    aps.app.onCancel = function () {
        aps.apsc.gotoView(common.getRelatedViewId('list'));
    };
});

