/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "aps/load",
    "aps/Memory",
    "aps/ResourceStore",
    "aps/xhr",
    "dojox/mvc/at",
    "dojox/mvc/getPlainValue",
    "dojox/mvc/getStateful",
    "./js/common.js",
    "./js/messages.js",
    "./js/wizardData.js"
], function (
    load,
    Memory,
    Store,
    xhr,
    at,
    getPlainValue,
    getStateful,
    common,
    messages,
    wizardData
) {
    var tenantId = aps.context.vars.tenant.aps.id,
        storagesStore = new Store({
            target: "/aps/2/resources/" + tenantId + "/storages"
        });

    function goToList() {
        aps.apsc.gotoView(common.getRelatedViewId('list'));
    }

    storagesStore.query().then(function (storages) {
        var servicePlan = wizardData.getWizardData(common.getRelatedViewId('edit')),
            model = getStateful({parameters: {}}),
            storagesOptions = storages.map(function (storage) {
                return {
                    id: storage.datacenterId + '_' + storage.id,
                    label: _("__storageName__ (__datacenterName__)", {
                        storageName: storage.name,
                        datacenterName: storage.datacenterName
                    })
                };
            });

        if (servicePlan.datacenterId && servicePlan.storageId) {
            model.set("parameters", {
                datacenterId: servicePlan.datacenterId,
                storageId: servicePlan.storageId
            });
        } else if (storages.length) {
            model.set("parameters", {
                datacenterId: storages[0].datacenterId,
                storageId: storages[0].id
            });
        }

        load(["aps/PageContainer", {id: "apsPageContainer"}, [
            ["aps/FieldSet", {id: common.genId('fieldset'),title: _("Assign storage")}, [
                ["aps/Output", {
                    id: common.genId('output'),
                    label: _("Service Plan name"),
                    value: servicePlan.name
                }],
                ["aps/Select", {
                    id: "storage",
                    label: _("Select storage"),
                    disabled: !storages.length,
                    description: _("Changing the storage for the plan will not impact the existing subscriptions."),
                    value: at(model, "parameters").transform({
                        format: function (value) {
                            if (value.datacenterId && value.storageId) {
                                return value.datacenterId + '_' + value.storageId;
                            }
                        },
                        parse: function (value) {
                            var parts = value.split('_');
                            return {
                                datacenterId: parts[0],
                                storageId: parts[1]
                            };
                        }
                    }),
                    store: new Memory({data: storagesOptions, idProperty: 'id'})
                }]
            ]]
        ]]).then(function () {
            aps.app.onSubmit = function () {
                xhr.put("/aps/2/resources/" + tenantId + "/plans/" + servicePlan.planId, {
                    data: JSON.stringify(getPlainValue(model.parameters)),
                    headers: {"Content-Type": "application/json"}
                }).then(goToList, messages.displayError);
            };
        });
    });

    aps.app.onCancel = goToList;
});