/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "dijit/registry",
    "dojo/when",
    "dojox/mvc/at",
    "dojox/mvc/getStateful",
    "aps/load",
    "aps/WizardData",
    "aps/Memory",
    "aps/CheckBox",
    "aps/TextBox",
    "aps/Output",
    "aps/ResourceStore",
    "aps/Container",
    "./js/messages.js",
    "./js/uomConverter.js",
    "./js/pcp-subscription/wizard/wizard.js",
    "./js/common.js"
], function (registry, when, at, getStateful, load, wd, Memory, CheckBox, TextBox, Output, Store, Container, messages, uomConverter, wizard, common) {

    var accountId = aps.context.params.accountid;    // 1000001;
    var templateId = aps.context.params.stemplateid; // "0445d988-d485-4b1a-842f-fb246b5fc934";

    var accountStore = new Store({target: "/aps/2/resources"});
    var templateStore = new Store({target: "/aps/2/resources"});
    var limitsStore = new Store({target: "/aps/2/resources/"+templateId+"/limits"});

    var accountModel, templateModel, subscriptionModel;


    accountStore.query("implementing(http://parallels.com/aps/types/pa/account/1.0),id=eq=" + accountId).then(function (accounts) {
        console.log(accounts);
        accountModel = getStateful(accounts[0]);

        templateStore.get(templateId).then(function (template) {
            console.log(template);
            templateModel = getStateful(template);
            limitsStore.query().then(function (objects) {
                subscriptionModel = getStateful({
                    subscriptionName: templateModel.name,
                    accountWideSU: false
                });

                var services = [], counters = [];

                var i, object;
                for(i=0; i<objects.length; ++i) {
                    object = getStateful(objects[i]);
                    if (common.isTenant(object.apsType) && object.hasOwnProperty('property')){
                        counters.push(object);
                    }
                    else {
                        services.push(object);
                    }
                }

                var countersStore = new Memory({ data: counters, idProperty: "apsId"});

                var columns = [
                    { "name": _("ID"),        "field": "apsId", sortable: false, visible: false },
                    { "name": _("Resource"),  "field": "title", sortable: false  },
                    { "name": _("Unlimited"), sortable: false,
                        renderCell: function (row) {
                            var unlimited = !row.hasOwnProperty("limit");
                            return new CheckBox({
                                id: "checkBox-"+row.id,
                                value: row.id,
                                checked: unlimited,
                                onClick: function(e){
                                    var id = this.get('value');
                                    var unlimited = e.target.checked;

                                    var textBox = registry.byId("textBox-"+id);
                                    textBox.set({
                                        disabled:  unlimited,
                                        required: !unlimited
                                    });
                                }
                            });
                        }
                    },
                    { "name": _("Limit"), "field": "limit", sortable: false ,
                        renderCell: function (row) {
                            var value, unlimited = !row.hasOwnProperty("limit");

                            if (unlimited) {
                                value = "";
                            }
                            else {
                                value = row.unit === "unit" ? row.limit : uomConverter.convert(row.limit, row.unit, "GB");
                            }

                            var limit = new TextBox({
                                id: "textBox-"+row.id,
                                value: value,
                                disabled: unlimited,
                                required: !unlimited,
                                "class": "f-limits",
                                autoSize: false,
                                pattern:  "^[0-9]+$",
                                missingMessage: _("This value is required.")
                            });

                            var unit = new Output({
                                value: row.unit === "unit" ? _(row.unit) : _("GB")
                            });

                            var limitContainer = new Container({id: "limit-"+row.id});
                            limitContainer.addChild(limit);
                            limitContainer.addChild(unit);

                            return limitContainer;
                        }
                    }
                ];

                load(["aps/PageContainer", { id: "apsPageContainer" }, [
                        wizard.getWizardControlView(wizard.VIEWS.PARAMETERS),
                        ["aps/FieldSet", {
                            id: common.genId('fieldset'),
                            description: _(
                                "You are ready to create a new subscription. " +
                                "Check the data below, edit if required and proceed.",
                                {linebreak: "<br/>"})
                            },
                            [
                                ["aps/Container", {id: common.genId('container'), label: _("Subscription Name")},
                                [["aps/TextBox", {id: common.genId('textbox'), value: at(subscriptionModel,"subscriptionName"), required: true }]]],
                                ["aps/Container", {id: common.genId('container'), label: _("Account")},           [["aps/Output",  { value: accountModel.companyName}]]],
                                ["aps/Container", {id: common.genId('container'), label: _("Service Template")},  [["aps/Output",  { value: templateModel.name}]]],
                                ["aps/Container", {id: common.genId('container'), label: _("Autoprovisioning")},  [["aps/Output",  {
                                    innerHTML: templateModel.autoprovisioning ? ("<img src='images/statuses/yes16.gif'/> "+_("Yes")) : ("<img src='images/statuses/no16.gif'/> "+_("No"))
                                }]]]
                            ]
                        ],
                        ["aps/Grid", {
                            id: common.genId('grid'),
                            columns: columns,
                            showPaging: false,
                            store: countersStore
                        }]
                    ]
                ]).then(function (){
                    aps.app.onSubmit = function () {
                        var page = registry.byId("apsPageContainer");
                        if (!page.validate()) {
                            aps.apsc.cancelProcessing();
                            return;
                        }

                        var id, limit, i, object,
                            resources = [];
                        for(i=0; i < services.length; ++i){
                            object = services[i];
                            id = parseInt(object.id, 10);
                            limit = object.hasOwnProperty("limit") ? object.limit : -1;
                            resources.push({
                                id:    id,
                                limit:  limit
                            });
                        }

                        for(i=0; i < counters.length; ++i){
                            object =  counters[i];
                            id = parseInt(object.id, 10);
                            var checkBox = registry.byId("checkBox-"+id);
                            if (checkBox.get('checked')) {
                                limit = -1;
                            }
                            else {
                                var textBox = registry.byId("textBox-"+id);
                                limit = parseInt(textBox.get("value"), 10);
                                if (object.unit!=="unit"){
                                    limit = uomConverter.convert(limit, "GB", object.unit);
                                }
                            }
                            resources.push({
                                id:       id,
                                limit:  limit
                            });
                        }

                        wd.put(null);
                        aps.apsc.gotoView(
                            aps.context.params.submitScreenView,
                            null,
                            {
                                subscriptionName:subscriptionModel.subscriptionName,
                                accountWideSU:subscriptionModel.accountWideSU,
                                resources: resources
                            }
                        );
                    };
                }).otherwise(messages.displayError);
            }).otherwise(messages.displayError);
        }).otherwise(messages.displayError);
    }).otherwise(messages.displayError);

    aps.app.onCancel = function () {
        wd.put(null);
        aps.apsc.gotoView(aps.context.params.cancelScreenView);
    };

    aps.app.onPrev = function () {
        aps.apsc.gotoView(aps.context.params.firstScreenView);
    };
});


