<?php
/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Base;

use Acronis\App\Base\ApplicationCounter as Counter;
use Arr;
use Rest\RestException;
use Str;

trait DependenciesTrait
{
    /**
     * @param Counter $counter
     * @param array $limits
     * @throws RestException
     */
    protected function checkCounterDependencies($counter, array $limits)
    {
        // do not check dependencies for resellers
        if ($this->tenant->type === \tenant::TYPE_RESELLER) {
            return;
        }
        $counterLimit = Arr::get($limits, $counter->getName(), 0);
        if (!$counterLimit) {
            return;
        }

        foreach ($counter->getDependencies() as $requiredCounterName) {
            $limit = (int)Arr::get($limits, $requiredCounterName, 0);
            if ($limit === 0) {
                throw new RestException(424, Str::format(
                    'Unable to provision resource "%s" without "%s". Please update the service template.',
                    $counter->getName(), $requiredCounterName
                ));
            }
        }
    }
}