<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Console\Command;

use Acronis\App\Services\Analytics\Google\MeasurementProtocol;
use Acronis\App\Services\Analytics\Monitor\APSSnapshot;
use Acronis\App\Services\Analytics\Monitor\Monitor;
use Acronis\App\Services\Analytics\Monitor\ProfileSnapshot;
use Acronis\App\Services\Analytics\Monitor\SubscriptionSnapshot;
use Acronis\App\Services\Apsc\ApscInstanceFactoryTrait;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

final class GoogleAnalytics extends Loggable
{
    use ApscInstanceFactoryTrait;
    /**
     * @return string
     */
    protected function getCommandName()
    {
        return 'analytics:send';
    }

    /**
     * @return string
     */
    protected function getCommandHelp()
    {
        return '<info>Send APS info to Google Analytics.</info>';
    }

    /**
     * @return string
     */
    protected function getCommandDescription()
    {
        return 'Send APS info to Google Analytics.';
    }

    /* ====== Main business logic of this command ====== */

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @return int|null|void
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $monitor = new Monitor([
            new APSSnapshot(),
            new ProfileSnapshot(),
            new SubscriptionSnapshot()
        ]);

        $data = $monitor->getData();
        $this->sendAnalytics($data);
        $output->writeln('APS info sent to Google Analytics successfully.');
    }


    /**
     * @throws \Exception
     */
    private function sendAnalytics($data)
    {
        $analytics = new MeasurementProtocol();
        $this->getLog()->notice(
            "Sending google analytic info:\n \"%s\"",
            var_export($data, true)
        );
        foreach ($data as $category => $value) {
            $this->sendAnalyticsValue($analytics, $category, $value);
        }
    }

    /**
     * @param MeasurementProtocol $analytics
     * @param $category
     * @param $value
     * @throws Exception
     */
    private function sendAnalyticsValue(MeasurementProtocol $analytics, $category, $value)
    {
        if (is_array($value)) {
            foreach ($value as $subcategory => $nestedValue) {
                $fullCategory = $category . ':' . $subcategory;
                $this->sendAnalyticsValue($analytics, $fullCategory, $nestedValue);
            }
            return;
        }
        $isValid = filter_var($value, FILTER_VALIDATE_INT, ['options' => ['min_range' => 0]]) !== false;
        if ($isValid) {
            $analytics->sendEvent($category, $value);
        } else {
            $analytics->sendEvent($category, 1, $value);
        }
    }
}
