<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Model_ConnectionChecker_OpenApiRequest implements Model_ConnectionChecker_CheckerInterface
{
    const ROOT_ACCOUNT_ID = 1;

    /** @var  Service_OpenApi_ApiInterface */
    private $_openApi;
    /** @var int */
    private $_errorCode = 0;
    /** @var string */
    private $_errorMessage = '';

    /**
     * @param Service_OpenApi_ApiInterface $openApi
     */
    public function __construct(Service_OpenApi_ApiInterface $openApi)
    {
        $this->_openApi = $openApi;
    }

    /**
     * @return bool
     */
    public function check()
    {
        $this->_errorCode = 0;
        $this->_errorMessage = '';

        try
        {
            $this->_openApi->getAccountInfo(self::ROOT_ACCOUNT_ID);
            return true;
        }
        catch(Exception $e)
        {
            $this->_errorCode = $e->getCode();
            $this->_errorMessage = $e->getMessage();
            return false;
        }
    }

    /**
     * @return array
     */
    public function getCheckDescription()
    {
        return (object) array(
            self::MESSAGE => _('Making test API call'),
            self::KEYS    => (object) array()
        );
    }

    /**
     * @return array
     */
    public function getCheckError()
    {
        return (object) array(
            self::CODE    => $this->_errorCode,
            self::MESSAGE => $this->_errorMessage
        );
    }
}