<?php
/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

use Acronis\Cloud\Client\Model\Common\Types\UserNotification;

class Model_PropertyConverters_NotificationConverter
{
    const BACKUP_NOTIFICATION_MASK_FAILED = 1;
    const BACKUP_NOTIFICATION_MASK_SUCCEEDED_WITH_ERRORS = 2;
    const BACKUP_NOTIFICATION_MASK_SUCCEEDED = 4;

    private static $backupNotificationsMasksToValuesMap = [
        self::BACKUP_NOTIFICATION_MASK_FAILED => UserNotification::BACKUP_ERROR,
        self::BACKUP_NOTIFICATION_MASK_SUCCEEDED_WITH_ERRORS => UserNotification::BACKUP_WARNING,
        self::BACKUP_NOTIFICATION_MASK_SUCCEEDED => UserNotification::BACKUP_INFO,
    ];

    private static $allManagementNotifications = [
        UserNotification::QUOTA,
        UserNotification::REPORTS,
    ];

    /**
     * @param int $backupNotifications
     * @param bool $enableManagementNotifications
     * @return string[]
     */
    public static function getNotifications($backupNotifications, $enableManagementNotifications)
    {
        return array_merge(
            static::buildNotificationsList($backupNotifications),
            $enableManagementNotifications ? static::$allManagementNotifications : []
        );
    }

    /**
     * @param $notifications int
     * @return string[]
     */
    private static function buildNotificationsList($notifications)
    {
        $notificationsList = [];

        foreach (static::$backupNotificationsMasksToValuesMap as $mask => $notification) {
            if (($notifications & $mask) !== 0) {
                $notificationsList[] = $notification;
            }
        }

        return $notificationsList;
    }
}