<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

use Acronis\App\Services\CountersManager\CountersManagerAwareTrait;

/**
 * Class Model_Resource_Tenant_Customer
 */
trait Model_Resource_DefaultUserInitializationTrait
{
    use Model_Resource_SubscriptionResourcesTrait;

    protected function initUserFormProfile($user)
    {
        $backupNotification = Enum_NotificationsBackup::DISABLE_NOTIFICATIONS;
        $managementNotification = false;

        $profile = $this->tenant->profile;
        if (!$profile->isEmpty()) {
            $backupNotification = $profile->backupNotification;
            $managementNotification = (bool)$profile->managementNotification;
        }

        $user->notifyBackup = $backupNotification;
        $user->notificationsManagement = $managementNotification;

        foreach ($this->fetchSubscriptionCounters() as $counter) {
            if ($this->hasUnlimitedUserQuota()) {
                $limit = self::UNLIMITED_VALUE;
            } else {
                $limit = Arr::getNestedItem($this->tenant, [$counter->getName(), 'limit'], 0);
            }

            $allOfferingItems = array_merge([$counter->getOfferingItemName()], $counter->getPartnerOfferingItems());
            foreach ($allOfferingItems as $offeringItemName) {
                $userOfferingItem = new UserOfferingItem();
                $userOfferingItem->name = $offeringItemName;
                $userOfferingItem->limit = $limit;
                $user->offeringItems[] = $userOfferingItem;
            }
        }
    }

    protected function hasUnlimitedUserQuota()
    {
        $profile = $this->tenant->profile;
        if ($profile->isEmpty()) {
            return false;
        }
        $userQuota = $profile->userQuota;

        return $userQuota === Service_APSApi_Api::PROFILE_USER_QUOTA_ANY ||
            $userQuota === Service_APSApi_Api::PROFILE_USER_QUOTA_OFF;
    }
}