<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

class Model_UserScreenNotificationManager implements Model_UserScreenNotificationManagerInterface
{
    const USER_PROVISION_NOTIFICATION_ID = "USER_PROVISION_NOTIFICATION_ID";

    const PROVISION_START_MESSAGE = 'PROVISION_START_MESSAGE';
    const PROVISION_START_DETAILS = 'PROVISION_START_DETAILS';
    const PROVISION_FINISH_MESSAGE = 'PROVISION_FINISH_MESSAGE';
    const PROVISION_FINISH_DETAILS = 'PROVISION_FINISH_DETAILS';
    const PROVISION_FAIL_MESSAGE = 'PROVISION_FAIL_MESSAGE';
    const PROVISION_FAIL_DETAILS = 'PROVISION_FAIL_DETAILS';
    const PROVISION_FAIL_SUBSCRIPTION_ALREADY_ASSIGNED_DETAILS = 'PROVISION_FAIL_SUBSCRIPTION_ALREADY_ASSIGNED_DETAILS';

    const CONFIGURE_START_MESSAGE = 'CONFIGURE_START_MESSAGE';
    const CONFIGURE_START_DETAILS = 'CONFIGURE_START_DETAILS';
    const CONFIGURE_FINISH_MESSAGE = 'CONFIGURE_FINISH_MESSAGE';
    const CONFIGURE_FINISH_DETAILS = 'CONFIGURE_FINISH_DETAILS';
    const CONFIGURE_FAIL_MESSAGE = 'CONFIGURE_FAIL_MESSAGE';
    const CONFIGURE_FAIL_DETAILS = 'CONFIGURE_FAIL_DETAILS';

    const UNPROVISION_START_MESSAGE = 'UNPROVISION_START_MESSAGE';
    const UNPROVISION_START_DETAILS = 'UNPROVISION_START_DETAILS';
    const UNPROVISION_FINISH_MESSAGE = 'UNPROVISION_FINISH_MESSAGE';
    const UNPROVISION_FINISH_DETAILS = 'UNPROVISION_FINISH_DETAILS';
    const UNPROVISION_FAIL_MESSAGE = 'UNPROVISION_FAIL_MESSAGE';
    const UNPROVISION_FAIL_DETAILS = 'UNPROVISION_FAIL_DETAILS';

    const ACTIVATE_ACCOUNT_START_MESSAGE = 'ACTIVATE_ACCOUNT_START_MESSAGE';
    const ACTIVATE_ACCOUNT_START_DETAILS = 'ACTIVATE_ACCOUNT_START_DETAILS';
    const ACTIVATE_ACCOUNT_FINISH_MESSAGE = 'ACTIVATE_ACCOUNT_FINISH_MESSAGE';
    const ACTIVATE_ACCOUNT_FINISH_DETAILS = 'ACTIVATE_ACCOUNT_FINISH_DETAILS';
    const ACTIVATE_ACCOUNT_FAIL_MESSAGE = 'ACTIVATE_ACCOUNT_FAIL_MESSAGE';
    const ACTIVATE_ACCOUNT_FAIL_DETAILS = 'ACTIVATE_ACCOUNT_FAIL_DETAILS';

    const CHANGE_PASSWORD_START_MESSAGE = 'CHANGE_PASSWORD_START_MESSAGE';
    const CHANGE_PASSWORD_START_DETAILS = 'CHANGE_PASSWORD_START_DETAILS';
    const CHANGE_PASSWORD_FINISH_MESSAGE = 'CHANGE_PASSWORD_FINISH_MESSAGE';
    const CHANGE_PASSWORD_FINISH_DETAILS = 'CHANGE_PASSWORD_FINISH_DETAILS';
    const CHANGE_PASSWORD_FAIL_MESSAGE = 'CHANGE_PASSWORD_FAIL_MESSAGE';
    const CHANGE_PASSWORD_FAIL_DETAILS = 'CHANGE_PASSWORD_FAIL_DETAILS';

    const LOG_EVENT_METHOD_WAS_CALLED = "Method \"%s\" was called.";
    const LOG_ERROR_SEND_NOTIFICATION = "Can't send notification. Error: %s";
    const LOG_ERROR_EDIT_NOTIFICATION = "Can't edit notification. Error: %s";
    const LOG_ERROR_DELETE_NOTIFICATION = "Can't delete notification. Error: %s";

    const SERVICE_NAME = 'Cyber Protection';

    const PLACEHOLDER_NAME = 'name';
    const PLACEHOLDER_LOGIN = 'login';
    const PLACEHOLDER_CURRENT_SUBSCRIPTION = 'currentSubscription';
    const PLACEHOLDER_ASSIGNED_SUBSCRIPTION = 'assignedSubscription';

    const NOTIFICATION_TTL = 31536000; // 60 * 60 * 24 * 365

    /**
     * @var users
     */
    private $_backupAccount;

    /**
     * @var string
     */
    private $_packageId;

    /**
     * @var string
     */
    private $_accountId;

    /**
     * @var string
     */
    private $_service;

    /**
     * @var string
     */
    private $_userId;

    /**
     * @var string
     */
    private $_initiatorId;

    /**
     * @var string
     */
    private $_displayName;

    /**
     * @var string
     */
    private $_endUserLogin;

    /**
     * @var Model_ScreenNotification_TransportInterface
     */
    private $_transport;

    /**
     * @var string
     */
    private $_unprovisionNotificationId;

    /**
     * @var string
     */
    private $_configureNotificationId;

    /**
     * @var string
     */
    private $_activateAccountNotificationId;

    /**
     * @var string
     */
    private $_changePasswordNotificationId;

    /**
     * @var array
     */
    private $_texts;

    /**
     * Model_UserScreenNotificationManager constructor.
     * @param users $backupAccount
     * @param Model_ScreenNotification_TransportInterface|null $transport
     */
    public function __construct(
        $backupAccount,
        Model_ScreenNotification_TransportInterface $transport = null
    )
    {
        $userId = $this->_getApsIdentityId();
        $this->_packageId = $backupAccount->aps->package->id;
        $this->_accountId = $backupAccount->tenant->account->aps->id;
        $this->_service = self::SERVICE_NAME;
        $this->_userId = $userId;
        $this->_initiatorId = $userId;
        $this->_displayName = $backupAccount->displayName;
        $this->_endUserLogin = $backupAccount->endUserLogin;
        $this->_backupAccount = $backupAccount;
        $this->_transport = $transport ?? new Model_ScreenNotification_Transport();
        $this->_texts = $this->_initTexts();
    }

    /**
     * @return string|null
     */
    private static function _getApsIdentityId()
    {
        return isset($_SERVER['HTTP_APS_IDENTITY_ID']) ? $_SERVER['HTTP_APS_IDENTITY_ID'] : NULL;
    }

    public function provisionHasStarted()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setPackageId($this->_packageId)
            ->setAccountId($this->_accountId)
            ->setService($this->_service)
            ->setUserId($this->_userId)
            ->setInitiatorId($this->_initiatorId)
            ->setType(Model_ScreenNotification_Struct_Message::TYPE_ACTIVITY)
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_IN_PROGRESS)
            ->setMessage($this->_getText(self::PROVISION_START_MESSAGE))
            ->setDetails($this->_getText(self::PROVISION_START_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $notification = $this->_sendNotification($notification);
        if (is_null($notification))
        {
            return;
        }
        $this->_setProvisionNotificationId($notification->getId());
    }

    public function provisionHasFinished()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        $provisionNotificationId = $this->_getProvisionNotificationId();
        if (is_null($provisionNotificationId))
        {
            return;
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_READY)
            ->setMessage($this->_getText(self::PROVISION_FINISH_MESSAGE))
            ->setDetails($this->_getText(self::PROVISION_FINISH_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $this->_editNotification($provisionNotificationId, $notification);
        $this->_resetProvisionNotificationId();
    }

    public function provisionHasFailed()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        $provisionNotificationId = $this->_getProvisionNotificationId();
        if (is_null($provisionNotificationId)) {
            return;
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_ERROR)
            ->setMessage($this->_getText(self::PROVISION_FAIL_MESSAGE))
            ->setDetails(
                $this->_getText(self::PROVISION_FAIL_DETAILS),
                array(
                    self::PLACEHOLDER_NAME => $this->_displayName,
                    self::PLACEHOLDER_LOGIN => $this->_endUserLogin
                )
            )
            ->getNotification();

        $this->_editNotification($provisionNotificationId, $notification);
        $this->_resetProvisionNotificationId();
    }

    /**
     * @param int $currentSubscriptionId
     * @param int $assignedSubscriptionId
     */
    public function provisionHasFailedSubscriptionAlreadyAssigned($currentSubscriptionId, $assignedSubscriptionId)
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        $provisionNotificationId = $this->_getProvisionNotificationId();
        if (is_null($provisionNotificationId)) {
            return;
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_ERROR)
            ->setMessage($this->_getText(self::PROVISION_FAIL_MESSAGE))
            ->setDetails($this->_getText(self::PROVISION_FAIL_SUBSCRIPTION_ALREADY_ASSIGNED_DETAILS), array(
                self::PLACEHOLDER_CURRENT_SUBSCRIPTION => $currentSubscriptionId,
                self::PLACEHOLDER_NAME => $this->_displayName,
                self::PLACEHOLDER_ASSIGNED_SUBSCRIPTION => $assignedSubscriptionId,
            ))
            ->getNotification();

        $this->_editNotification($provisionNotificationId, $notification);
        $this->_resetProvisionNotificationId();
    }

    public function unprovisionHasStarted()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        $provisionNotificationId = $this->_getProvisionNotificationId();
        if (!is_null($provisionNotificationId))
        {
            $this->_deleteNotification($provisionNotificationId);
            $this->_resetProvisionNotificationId();
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setPackageId($this->_packageId)
            ->setAccountId($this->_accountId)
            ->setService($this->_service)
            ->setUserId($this->_userId)
            ->setInitiatorId($this->_initiatorId)
            ->setType(Model_ScreenNotification_Struct_Message::TYPE_ACTIVITY)
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_IN_PROGRESS)
            ->setMessage($this->_getText(self::UNPROVISION_START_MESSAGE))
            ->setDetails($this->_getText(self::UNPROVISION_START_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $notification = $this->_sendNotification($notification);
        if (is_null($notification))
        {
            return;
        }
        $this->_unprovisionNotificationId = $notification->getId();
    }

    public function unprovisionHasFinished()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        if (is_null($this->_unprovisionNotificationId))
        {
            return;
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_READY)
            ->setMessage($this->_getText(self::UNPROVISION_FINISH_MESSAGE))
            ->setDetails($this->_getText(self::UNPROVISION_FINISH_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $this->_editNotification($this->_unprovisionNotificationId, $notification);
        $this->_unprovisionNotificationId = null;
    }

    public function unprovisionHasFailed()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        if (is_null($this->_unprovisionNotificationId))
        {
            return;
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_ERROR)
            ->setMessage($this->_getText(self::UNPROVISION_FAIL_MESSAGE))
            ->setDetails($this->_getText(self::UNPROVISION_FAIL_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $this->_editNotification($this->_unprovisionNotificationId, $notification);
        $this->_unprovisionNotificationId = null;
    }

    public function configureHasStarted()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setPackageId($this->_packageId)
            ->setAccountId($this->_accountId)
            ->setService($this->_service)
            ->setUserId($this->_userId)
            ->setInitiatorId($this->_initiatorId)
            ->setType(Model_ScreenNotification_Struct_Message::TYPE_ACTIVITY)
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_IN_PROGRESS)
            ->setMessage($this->_getText(self::CONFIGURE_START_MESSAGE))
            ->setDetails($this->_getText(self::CONFIGURE_START_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $notification = $this->_sendNotification($notification);
        if (is_null($notification))
        {
            return;
        }
        $this->_configureNotificationId = $notification->getId();
    }

    public function configureHasFinished()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        if (is_null($this->_configureNotificationId))
        {
            return;
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_READY)
            ->setMessage($this->_getText(self::CONFIGURE_FINISH_MESSAGE))
            ->setDetails($this->_getText(self::CONFIGURE_FINISH_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $this->_editNotification($this->_configureNotificationId, $notification);
        $this->_configureNotificationId = null;
    }

    public function configureHasFailed()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        if (is_null($this->_configureNotificationId))
        {
            return;
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_ERROR)
            ->setMessage($this->_getText(self::CONFIGURE_FAIL_MESSAGE))
            ->setDetails($this->_getText(self::CONFIGURE_FAIL_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $this->_editNotification($this->_configureNotificationId, $notification);
        $this->_configureNotificationId = null;
    }

    public function activateAccountHasStarted()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setPackageId($this->_packageId)
            ->setAccountId($this->_accountId)
            ->setService($this->_service)
            ->setUserId($this->_userId)
            ->setInitiatorId($this->_initiatorId)
            ->setType(Model_ScreenNotification_Struct_Message::TYPE_ACTIVITY)
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_IN_PROGRESS)
            ->setMessage($this->_getText(self::ACTIVATE_ACCOUNT_START_MESSAGE))
            ->setDetails($this->_getText(self::ACTIVATE_ACCOUNT_START_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $notification = $this->_sendNotification($notification);
        if (is_null($notification))
        {
            return;
        }
        $this->_activateAccountNotificationId = $notification->getId();
    }

    public function activateAccountHasFinished()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        if (is_null($this->_activateAccountNotificationId))
        {
            return;
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_READY)
            ->setMessage($this->_getText(self::ACTIVATE_ACCOUNT_FINISH_MESSAGE))
            ->setDetails($this->_getText(self::ACTIVATE_ACCOUNT_FINISH_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $this->_editNotification($this->_activateAccountNotificationId, $notification);
        $this->_activateAccountNotificationId = null;
    }

    public function activateAccountHasFailed()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        if (is_null($this->_activateAccountNotificationId))
        {
            return;
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_ERROR)
            ->setMessage($this->_getText(self::ACTIVATE_ACCOUNT_FAIL_MESSAGE))
            ->setDetails($this->_getText(self::ACTIVATE_ACCOUNT_FAIL_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $this->_editNotification($this->_activateAccountNotificationId, $notification);
        $this->_activateAccountNotificationId = null;
    }

    public function changePasswordHasStarted()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setPackageId($this->_packageId)
            ->setAccountId($this->_accountId)
            ->setService($this->_service)
            ->setUserId($this->_userId)
            ->setInitiatorId($this->_initiatorId)
            ->setType(Model_ScreenNotification_Struct_Message::TYPE_ACTIVITY)
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_IN_PROGRESS)
            ->setMessage($this->_getText(self::CHANGE_PASSWORD_START_MESSAGE))
            ->setDetails($this->_getText(self::CHANGE_PASSWORD_START_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $notification = $this->_sendNotification($notification);
        if (is_null($notification))
        {
            return;
        }
        $this->_changePasswordNotificationId = $notification->getId();
    }

    public function changePasswordHasFinished()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        if (is_null($this->_changePasswordNotificationId))
        {
            return;
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_READY)
            ->setMessage($this->_getText(self::CHANGE_PASSWORD_FINISH_MESSAGE))
            ->setDetails($this->_getText(self::CHANGE_PASSWORD_FINISH_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $this->_editNotification($this->_changePasswordNotificationId, $notification);
        $this->_changePasswordNotificationId = null;
    }

    public function changePasswordHasFailed()
    {
        Log::getInstance()->event(sprintf(self::LOG_EVENT_METHOD_WAS_CALLED, __METHOD__));

        if (is_null($this->_changePasswordNotificationId))
        {
            return;
        }

        $builder = new Model_ScreenNotification_Builder();
        $notification = $builder
            ->setStatus(Model_ScreenNotification_Struct_Message::STATUS_ERROR)
            ->setMessage($this->_getText(self::CHANGE_PASSWORD_FAIL_MESSAGE))
            ->setDetails($this->_getText(self::CHANGE_PASSWORD_FAIL_DETAILS), array(self::PLACEHOLDER_NAME => $this->_displayName))
            ->getNotification();

        $this->_editNotification($this->_changePasswordNotificationId, $notification);
        $this->_changePasswordNotificationId = null;
    }

    /**
     * @param $notification
     * @return Model_ScreenNotification_Struct_MessageInterface|null
     */
    private function _sendNotification($notification)
    {
        try
        {
            return $this->_transport->send($notification);
        }
        catch(Exception $e)
        {
            Log::getInstance()->error(sprintf(self::LOG_ERROR_SEND_NOTIFICATION, $e->getMessage()));
        }
        return null;
    }

    /**
     * @param $notificationId
     * @param $notification
     * @return Model_ScreenNotification_Struct_MessageInterface|null
     */
    private function _editNotification($notificationId, $notification)
    {
        try
        {
            return $this->_transport->edit($notificationId, $notification);
        }
        catch(Exception $e)
        {
            Log::getInstance()->error(sprintf(self::LOG_ERROR_EDIT_NOTIFICATION, $e->getMessage()));
        }
        return null;
    }

    /**
     * @param $notificationId
     * @return null|string
     */
    private function _deleteNotification($notificationId)
    {
        try
        {
            return $this->_transport->delete($notificationId);
        }
        catch(Exception $e)
        {
            Log::getInstance()->error(sprintf(self::LOG_ERROR_DELETE_NOTIFICATION, $e->getMessage()));
        }
        return null;
    }

    /**
     * @return mixed
     */
    private function _getProvisionNotificationId()
    {
        return $this->_backupAccount->getValueFromCache(self::USER_PROVISION_NOTIFICATION_ID);
    }

    /**
     * @param $notificationId
     */
    private function _setProvisionNotificationId($notificationId)
    {
        $this->_backupAccount->setValueToCache(self::USER_PROVISION_NOTIFICATION_ID, $notificationId, self::NOTIFICATION_TTL);
    }

    private function _resetProvisionNotificationId()
    {
        $this->_backupAccount->unsetValueFromCache(self::USER_PROVISION_NOTIFICATION_ID);
    }

    /**
     * @param $key
     * @return null
     */
    private function _getText($key)
    {
        return isset($this->_texts[$key]) ? $this->_texts[$key] : null;
    }

    /**
     * @return array
     */
    private function _initTexts()
    {
        return array(
            self::PROVISION_START_MESSAGE  => _("Assigning the cyber protection service"),
            self::PROVISION_START_DETAILS  => _("Assigning the cyber protection service to the user \"__name__\"."),
            self::PROVISION_FINISH_MESSAGE => _("The cyber protection service has been assigned"),
            self::PROVISION_FINISH_DETAILS => _("The cyber protection service has been successfully assigned to the user \"__name__\"."),
            self::PROVISION_FAIL_MESSAGE   => _("Failed to assign the cyber protection service"),
            self::PROVISION_FAIL_DETAILS   => _("Failed to assign the cyber protection service to the user \"__name__\". The user with email address \"__login__\" already exists in the cyber protection console. Delete the existing user or use a different email."),
            self::PROVISION_FAIL_SUBSCRIPTION_ALREADY_ASSIGNED_DETAILS   => _("Failed to assign the cyber protection service from subscription __currentSubscription__ to user \"__name__\" because it is already assigned from subscription __assignedSubscription__."),

            self::CONFIGURE_START_MESSAGE  => _("Updating the cyber protection service settings"),
            self::CONFIGURE_START_DETAILS  => _("Updating the cyber protection service settings for the user \"__name__\"."),
            self::CONFIGURE_FINISH_MESSAGE => _("The cyber protection service settings have been updated"),
            self::CONFIGURE_FINISH_DETAILS => _("The cyber protection service settings have been successfully updated for the user \"__name__\"."),
            self::CONFIGURE_FAIL_MESSAGE   => _("Failed to update the cyber protection service settings"),
            self::CONFIGURE_FAIL_DETAILS   => _("Failed to update the cyber protection service settings for the user \"__name__\"."),

            self::UNPROVISION_START_MESSAGE  => _("Unassigning the cyber protection service"),
            self::UNPROVISION_START_DETAILS  => _("Unassigning the cyber protection service from the user \"__name__\"."),
            self::UNPROVISION_FINISH_MESSAGE => _("The cyber protection service has been unassigned"),
            self::UNPROVISION_FINISH_DETAILS => _("The cyber protection service has been successfully unassigned from user \"__name__\"."),
            self::UNPROVISION_FAIL_MESSAGE   => _("Failed to unassign the cyber protection service"),
            self::UNPROVISION_FAIL_DETAILS   => _("Failed to unassign the cyber protection service from the user \"__name__\"."),

            self::ACTIVATE_ACCOUNT_START_MESSAGE  => _("Activating the cyber protection service"),
            self::ACTIVATE_ACCOUNT_START_DETAILS  => _("Activating the cyber protection service for the user \"__name__\"."),
            self::ACTIVATE_ACCOUNT_FINISH_MESSAGE => _("The cyber protection service has been activated"),
            self::ACTIVATE_ACCOUNT_FINISH_DETAILS => _("The cyber protection service has been successfully activated for the user \"__name__\"."),
            self::ACTIVATE_ACCOUNT_FAIL_MESSAGE   => _("Failed to activate the cyber protection service"),
            self::ACTIVATE_ACCOUNT_FAIL_DETAILS   => _("Failed to activate the cyber protection service for the user \"__name__\"."),

            self::CHANGE_PASSWORD_START_MESSAGE  => _("Changing the cyber protection service password"),
            self::CHANGE_PASSWORD_START_DETAILS  => _("Changing the cyber protection service password for the user \"__name__\"."),
            self::CHANGE_PASSWORD_FINISH_MESSAGE => _("The cyber protection service password has been changed"),
            self::CHANGE_PASSWORD_FINISH_DETAILS => _("The cyber protection service password has been successfully changed for user \"__name__\"."),
            self::CHANGE_PASSWORD_FAIL_MESSAGE   => _("Failed to change the cyber protection service password"),
            self::CHANGE_PASSWORD_FAIL_DETAILS   => _("Failed to change the cyber protection service password for the user \"__name__\"."),
        );
    }
}