<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_APSApi_Api implements Service_APSApi_ApiInterface
{
    const APS_READY = 'aps:ready';
    const APS_SUSPENDED = 'aps:suspended';

    const CLOUD_TYPE_LIKE = 'http%3A%2F%2Facronis.com%2Faps%2Fcloud-backup-v1%2F*';

    const RQL_ADMIN_USER = 'implementing(http://parallels.com/aps/types/pa/admin-user/1.0),sort(memberId),limit(0,1)';
    const RQL_STAFF_MEMBER = 'implementing(http://aps-standard.org/types/core/admin-user/1.0),eq(aps.id,%s)';
    const RQL_STAFF_MEMBERS = 'implementing(http://parallels.com/aps/types/pa/admin-user/1.0),sort(memberId)';
    const RQL_BACKUP_ADMIN = 'implementing(http://acronis.com/aps/cloud-backup-v1/admin/4.0),select(staffMember),eq(staffMember.aps.id,%s)';
    const RQL_PATH_USERS = '/aps/2/resources/%s/users/';
    const RQL_PATH_LIMITS = '/aps/2/resources/%s/limits';
    const RQL_PATH_CONFIGURATION_MANAGER = '/aps/2/services/configuration-manager/systemProperty?account_id=%s&name=%s';
    const RQL_STATUS_NOT_READY = 'like(aps.type,%s),out(aps.status,(%s))';
    const RQL_STATUS_NOT_READY_EXCLUDE = 'like(aps.type,%s),out(aps.status,(%s)),out(aps.id,(%s))';
    const RQL_SERVICE_TEMPLATE = "implementing(http://parallels.com/aps/types/pa/serviceTemplate/1.0)";
    const RQL_IMPLEMENTING_EQ_ACCOUNT_ID = "implementing(%s),eq(accountId,%s)";
    const RQL_IMPLEMENTING_EQ_ACCOUNT_ID_NE_APS_ID = "implementing(%s),eq(accountId,%s),ne(aps.id,%s)";
    const RQL_IMPLEMENTING_TENANT_FOR_GLOBAL_BY_VERSION = "implementing(http://acronis.com/aps/cloud-backup-v1/tenant/%s),eq(global.aps.id,%s)";
    const RQL_IMPLEMENTING_PROFILE_FOR_GLOBAL_BY_VERSION = "implementing(http://acronis.com/aps/cloud-backup-v1/backup-profile/%s),eq(global.aps.id,%s)";
    const RQL_IMPLEMENTING_TIERS_FOR_GLOBAL_BY_VERSION = "implementing(http://acronis.com/aps/cloud-backup-v1/minimal-commitment-tier/%s),eq(global.aps.id,%s)";
    const RQL_IMPLEMENTING_DATACENTER_FOR_GLOBAL_BY_VERSION = "implementing(http://acronis.com/aps/cloud-backup-v1/datacenter/%s),eq(global.aps.id,%s)";
    const RQL_IMPLEMENTING_USERS_FOR_TENANT_BY_VERSION = "implementing(http://acronis.com/aps/cloud-backup-v1/users/%s),eq(tenant.aps.id,%s)";
    const RQL_IMPLEMENTING_ADMINS_EQ_TENANT_APS_ID = "implementing(http://acronis.com/aps/cloud-backup-v1/admin/%s),eq(tenant.aps.id,%s)";

    const RQL_LIMIT_0 = '%s,limit(0,0)';

    const RESOURCE_VERSION_1_0 = '1.0';
    const RESOURCE_VERSION_2_0 = '2.0';
    const RESOURCE_VERSION_3_0 = '3.0';
    const RESOURCE_VERSION_4_0 = '4.0';
    const RESOURCE_VERSION_5_0 = '5.0';
    const RESOURCE_VERSION_6_0 = '6.0';
    const RESOURCE_VERSION_7_0 = '7.0';
    const RESOURCE_VERSION_8_0 = '8.0';
    const RESOURCE_VERSION_9_0 = '9.0';

    const PROFILE_DEFAULT_NAME = "Default Service Profile (Acronis)";
    const PROFILE_DEFAULT_BACKUP_NOTIFICATION = 3;
    const PROFILE_DEFAULT_MANAGEMENT_NOTIFICATION = 0;
    const PROFILE_DEFAULT_USER_QUOTA = 1;
    const PROFILE_DEFAULT_GROUP_TYPE = 0;
    const PROFILE_DEFAULT_STORAGE_POLICY = 0;
    const PROFILE_DEFAULT_STORAGE_LOCATION = 1;

    const EX_MSG_ST_CONTAINS_SEVERAL_BACKUP_PROFILES = 'Only one Service Profile resource can be included into the service template.';
    const EX_MSG_LIMIT_OF_RESOURCE_DOES_NOT_ONE = 'The limit of the Service Profile resource in the service template must be equal to 1.';

    const PROFILE_USER_QUOTA_ANY = 2;
    const PROFILE_USER_QUOTA_LIMITS = 1;
    const PROFILE_USER_QUOTA_OFF = 0;

    const EXCEPTION_GROUP_ALREADY_ASSIGNED = 'The group with ID %s is already assigned to subscription %s.';
    const EXCEPTION_UNABLE_CALCULATE_RESOURCES = 'There is no ability to calculate count of the resources';
    const EXCEPTION_NO_STAFF_MEMBERS_EXCEPTION = 'There are no required admin staff members on Customer Account.';

    const REGEXP_PATTERN_GET_COUNT_ITEM = '#items \*/(\d+)#';
    const FIELD_HTTP_HEADER_CONTENT_RANGE = 'CONTENT_RANGE';

    const TENANT_TYPE = 'http://acronis.com/aps/cloud-backup-v1/tenant/';

    const APS_TYPES_PA_ACCOUNT_1_0 = 'http://parallels.com/aps/types/pa/account/1.0';
    const APS_TYPES_PA_SUBSCRIPTION_1_0 = 'http://parallels.com/aps/types/pa/subscription/1.0';

    /** @var \APS\ControllerProxy */
    private $_apsController;

    /**
     * @param \APS\ControllerProxy $apsController
     */
    public function __construct(\APS\ControllerProxy $apsController)
    {
        $this->_apsController = $apsController;
    }

    /**
     * @param RqlBuilder $rqlBuilder
     * @param $base
     * @return array
     */
    public function getAllResources(RqlBuilder $rqlBuilder, $base = null) {
        $limit = 1000;
        $offset = 0;

        $apsc = is_null($base) ? $this->_apsController : $this->_apsController->impersonate($base);
        $targetResources = array();
        do {
            $currentRqlBuilder = clone $rqlBuilder;
            $rql = $currentRqlBuilder->limit($offset, $limit)->getQuery();

            $resources = $apsc->getResources($rql);
            $targetResources = array_merge($targetResources, $resources);

            $offset += $limit;
        } while (count($resources) === $limit);
        return $targetResources;
    }

    /**
     * @param array $subscriptionResources
     * @throws Exception
     * @return \APS\ResourceBase
     */
    public function getProfile(array $subscriptionResources)
    {
        $profiles = array();
        $profileType = \APS\TypeLibrary::getTypeByServiceId('profile');
        $profileTypeWithoutVersion = substr($profileType, 0, strrpos($profileType, '/'));
        foreach ($subscriptionResources as $res) {
            if (!isset($res->apsType)) {
                continue;
            }
            if (strstr($res->apsType, $profileTypeWithoutVersion) !== false) {
                $profiles[] = $res;
            }
        }

        $profilesCount = count($profiles);
        if ($profilesCount > 1) {
            throw new Exception(self::EX_MSG_ST_CONTAINS_SEVERAL_BACKUP_PROFILES);
        }
        if ($profilesCount < 1) {
            return null;
        }

        $res = array_pop($profiles);
        if (!isset($res->limit) || $res->limit != 1) {
            throw new Exception(self::EX_MSG_LIMIT_OF_RESOURCE_DOES_NOT_ONE);
        }
        return $this->_apsController->getResource($res->apsId);
    }

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy|string $tenant
     * @throws Exception
     * @return \APS\ResourceBase|NULL
     */
    public function getAdmin($tenant)
    {
        $adminusers = $this->_apsController->impersonate($tenant)->getResources(self::RQL_ADMIN_USER, $tenant->account->users->getLink()->href);
        return !empty($adminusers) ? reset($adminusers) : NULL;
    }

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy $base
     * @param string $staffMemberId
     * @return \APS\ResourceBase|NULL
     */
    public function getStaffMember($base, $staffMemberId)
    {
        $rql = sprintf(self::RQL_STAFF_MEMBER, $staffMemberId);
        $staffMembers = $this->_apsController->impersonate($base)->getResources($rql);
        if (is_array($staffMembers) && count($staffMembers) === 1)
        {
            return reset($staffMembers);
        }
        return NULL;
    }

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy|string $base
     * @param \APS\ResourceBase|\APS\ResourceProxy $account
     * @param null|integer $limit
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getStaffMembers($base, $account, $limit = null)
    {
        $limitExpression = is_null($limit) ? '' : Str::format(',limit(0,%s)', $limit);

        return $this->_apsController->impersonate($base)->getResources(
            self::RQL_STAFF_MEMBERS . $limitExpression,
            Str::format(self::RQL_PATH_USERS, $account->aps->id)
        );
    }

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy $tenant
     * @param string $staffMemberId
     * @return \APS\ResourceBase|NULL
     */
    public function getBackupAdministrator($tenant, $staffMemberId)
    {
        $rql = sprintf(self::RQL_BACKUP_ADMIN, $staffMemberId);
        $admins = $this->_apsController->impersonate($tenant)->getResources($rql);

        if (is_array($admins) && count($admins) === 1) {
            return reset($admins);
        }

        return null;
    }

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy|string $base
     * @param string | array $status
     * @param string | array | null $excludeId
     * @return \APS\ResourceBase[]
     */
    public function getPackageResourcesByStatusNot($base, $status, $excludeId = null)
    {
        $apsc = $this->_apsController->impersonate($base);
        $rql = is_null($excludeId) ?
            sprintf(
                self::RQL_STATUS_NOT_READY,
                self::CLOUD_TYPE_LIKE,
                is_array($status) ? implode(',', $status) : $status
            ) :
            sprintf(
                self::RQL_STATUS_NOT_READY_EXCLUDE,
                self::CLOUD_TYPE_LIKE,
                is_array($status) ? implode(',', $status) : $status,
                is_array($excludeId) ? implode(',', $excludeId) : $excludeId
            );
        return $apsc->getResources($rql);
    }

    /**
     * @return \APS\ResourceBase[]
     * @throws Exception|\Rest\RestException
     */
    public function getServiceTemplatesWithLimits()
    {
        $templates = array();
        $availableTemplates = $this->_apsController->getResources(self::RQL_SERVICE_TEMPLATE);
        foreach($availableTemplates as $availableTemplate)
        {
            $limits = array();
            $availableLimits = json_decode($this->_apsController->getIo()->sendRequest(
                \APS\Proto::GET,
                sprintf(self::RQL_PATH_LIMITS, $availableTemplate->aps->id)
            ));
            foreach($availableLimits as $limit)
            {
                $limits[$limit->id] = $limit;
            }
            $availableTemplate->limits = $limits;
            $templates[$availableTemplate->serviceTemplateId] = $availableTemplate;
        }
        return $templates;
    }

    /**
     * @return \APS\ResourceBase[]
     * @throws Exception|\Rest\RestException
     */
    public function getPropertyForAccount($accountId, $name)
    {
        return json_decode($this->_apsController->getIo()->sendRequest(
            \APS\Proto::GET,
            sprintf(self::RQL_PATH_CONFIGURATION_MANAGER, $accountId, $name)
        ));
    }

    /**
     * @return array
     * @throws Exception
     */
    public function getApsVersions()
    {
        $response = json_decode($this->_apsController->getIo()->sendRequest(\APS\Proto::GET, '/aps'));

        return array_keys(get_object_vars($response->versions));
    }

    /**
     * @param bool $type
     * @return array
     * @throws Exception
     */
    public function getApplicationInstances($type = false)
    {
        // applications,implements() doesn't work in Odin8.2
        $response = json_decode($this->_apsController->getIo()->sendRequest(\APS\Proto::GET, '/aps/2/applications'));
        if (!$type) {
            return $response;
        }
        $result = [];
        foreach ($response as $appInstance) {
            if ($appInstance->aps->type == $type) {
                $result[] = $appInstance;
            }
        }

        return $result;
    }

    /**
     * @param int $accountId
     * @param string|null $excludeTenant
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getTenantsByAccountId($accountId, $excludeTenant = null)
    {
        $rql = is_null($excludeTenant)?
            sprintf(self::RQL_IMPLEMENTING_EQ_ACCOUNT_ID, \APS\TypeLibrary::getTypeByServiceId('tenant'), $accountId):
            sprintf(self::RQL_IMPLEMENTING_EQ_ACCOUNT_ID_NE_APS_ID, \APS\TypeLibrary::getTypeByServiceId('tenant'), $accountId, $excludeTenant);
        return $this->_apsController->getResources($rql);
    }

    /**
     * Receive tenants linked with global
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $global
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getTenantsForGlobal($global)
    {
        $tenants = array();
        $versions = array(
            Service_APSApi_Api::RESOURCE_VERSION_9_0,
            Service_APSApi_Api::RESOURCE_VERSION_8_0,
            Service_APSApi_Api::RESOURCE_VERSION_7_0,
            Service_APSApi_Api::RESOURCE_VERSION_6_0,
            Service_APSApi_Api::RESOURCE_VERSION_5_0
        );
        foreach ($versions as $version) {
            try {
                $rqlBuilder = RqlBuilder::createInstance()
                    ->implementing(self::TENANT_TYPE . $version)
                    ->eq('global.aps.id', $global->aps->id);
                $tenants = $this->getAllResources($rqlBuilder, $global);
            } catch (\Rest\RestException $e) {
                //We have exception because resources can be not exist
                // Skip this exception
            }
            if (!empty($tenants)) {
                break;
            }
        }
        return $tenants;
    }

    /**
     * Receive profiles linked with global
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $global
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getProfilesForGlobal($global)
    {
        $apsc = $this->_apsController->impersonate($global);
        $profiles = array();
        $versions = array(
            Service_APSApi_Api::RESOURCE_VERSION_5_0,
            Service_APSApi_Api::RESOURCE_VERSION_4_0,
            Service_APSApi_Api::RESOURCE_VERSION_3_0,
            Service_APSApi_Api::RESOURCE_VERSION_2_0,
            Service_APSApi_Api::RESOURCE_VERSION_1_0
        );

        foreach ($versions as $version) {
            try {
                $profiles = $apsc->getResources(sprintf(self::RQL_IMPLEMENTING_PROFILE_FOR_GLOBAL_BY_VERSION, $version, $global->aps->id));
            } catch (\Rest\RestException $e) {
                //We have exception because resources can be not exist
                // Skip this exception
            }
            if (!empty($profiles)) {
                break;
            }
        }

        return $profiles;
    }

    /**
     * Receive profiles linked with global
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $global
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getMinimalCommitmentTiersForGlobal($global)
    {
        $apsc = $this->_apsController->impersonate($global);
        $tiers = array();
        $versions = array(
            Service_APSApi_Api::RESOURCE_VERSION_4_0,
            Service_APSApi_Api::RESOURCE_VERSION_3_0,
            Service_APSApi_Api::RESOURCE_VERSION_2_0,
            Service_APSApi_Api::RESOURCE_VERSION_1_0
        );

        foreach ($versions as $version) {
            try {
                $tiers = $apsc->getResources(sprintf(self::RQL_IMPLEMENTING_TIERS_FOR_GLOBAL_BY_VERSION, $version, $global->aps->id));
            } catch (\Rest\RestException $e) {
                //We have exception because resources can be not exist
                // Skip this exception
            }
            if (!empty($tiers)) {
                break;
            }
        }

        return $tiers;
    }

    /**
     * Receive datacenters linked with global
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $global
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getDatacentersForGlobal($global)
    {
        $apsc = $this->_apsController->impersonate($global);
        $datacenters = array();
        $versions = array(
            Service_APSApi_Api::RESOURCE_VERSION_4_0,
            Service_APSApi_Api::RESOURCE_VERSION_3_0,
            Service_APSApi_Api::RESOURCE_VERSION_2_0,
            Service_APSApi_Api::RESOURCE_VERSION_1_0
        );

        foreach ($versions as $version) {
            try {
                $datacenters = $apsc->getResources(sprintf(self::RQL_IMPLEMENTING_DATACENTER_FOR_GLOBAL_BY_VERSION, $version, $global->aps->id));
            } catch (\Rest\RestException $e) {
                //We have exception because resources can be not exist
                // Skip this exception
            }
            if (!empty($datacenters)) {
                break;
            }
        }

        return $datacenters;
    }

    /**
     * Receive users linked with tenant
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $tenant
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getUsersForTenant($tenant)
    {
        $apsc = $this->_apsController->impersonate($tenant);
        $users = array();
        $versions = array(
            Service_APSApi_Api::RESOURCE_VERSION_7_0,
            Service_APSApi_Api::RESOURCE_VERSION_6_0,
            Service_APSApi_Api::RESOURCE_VERSION_5_0,
            Service_APSApi_Api::RESOURCE_VERSION_4_0,
            Service_APSApi_Api::RESOURCE_VERSION_3_0
        );

        foreach ($versions as $version) {
            try {
                $users = $apsc->getResources(sprintf(self::RQL_IMPLEMENTING_USERS_FOR_TENANT_BY_VERSION, $version, $tenant->aps->id));
            } catch (\Rest\RestException $e) {
                //We have exception because resources can be not exist
                // Skip this exception
            }
            if (!empty($users)) {
                break;
            }
        }

        return $users;
    }

    /**
     * Receive admins linked with tenant
     *
     * NOTE, this only works for PA version >= 6.0
     *
     * @param \APS\ResourceBase $tenant
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getAdminsForTenant($tenant)
    {
        $apsc = $this->_apsController->impersonate($tenant);
        $admins = array();
        $versions = array(
            Service_APSApi_Api::RESOURCE_VERSION_4_0,
            Service_APSApi_Api::RESOURCE_VERSION_3_0,
            Service_APSApi_Api::RESOURCE_VERSION_2_0,
            Service_APSApi_Api::RESOURCE_VERSION_1_0
        );
        foreach ($versions as $version) {
            try {
                $admins = $apsc->getResources(sprintf(self::RQL_IMPLEMENTING_ADMINS_EQ_TENANT_APS_ID, $version, $tenant->aps->id));
            } catch (\Rest\RestException $e) {
                //We have exception because resources can be not exist
                // Skip this exception
            }
            if (!empty($admins)) {
                break;
            }
        }
        return $admins;
    }

    /**
     * @param \APS\ResourceBase|\APS\ResourceProxy|string $globals
     */
    public function makeDefaultProfile($globals)
    {
        $apsc = $this->_apsController->impersonate($globals);

        $profile = \APS\TypeLibrary::newResourceByTypeId(\APS\TypeLibrary::getTypeByServiceId('profile'));
        $profile->global->set($globals);
        $profile->name = self::PROFILE_DEFAULT_NAME;
        $profile->backupNotification = self::PROFILE_DEFAULT_BACKUP_NOTIFICATION;
        $profile->managementNotification = self::PROFILE_DEFAULT_MANAGEMENT_NOTIFICATION;
        $profile->userQuota = self::PROFILE_DEFAULT_USER_QUOTA;
        $profile->mspGroupType = self::PROFILE_DEFAULT_GROUP_TYPE;
        $profile->storages = array();
        $profile->storagePolicy = self::PROFILE_DEFAULT_STORAGE_POLICY;

        $apsc->provisionResource($profile);
    }

    /**
     * @param int $groupId
     * @param int $datacenterId
     * @throws \Rest\RestException
     */
    public function checkGroupCanBeAssignedOrEx($groupId, $datacenterId = null)
    {
        $tenants = $this->getTenantsByGroupIdAndDatacenterId($groupId, $datacenterId);
        if (count($tenants) > 0) {
            throw new \Rest\RestException(409, sprintf(self::EXCEPTION_GROUP_ALREADY_ASSIGNED, $groupId, $tenants[0]->subscription->subscriptionId));
        }
    }

    /**
     * @param string $rql
     * @return int
     * @throws \Rest\RestException
     */
    public function countResources($rql, $base = null)
    {
        $rql = sprintf(self::RQL_LIMIT_0, $rql);
        $apsc = (is_null($base)) ? $this->_apsController : $this->_apsController->impersonate($base);

        $resurces = $apsc->getResources($rql);

        $contentRange = $apsc->getLastResponseHeader(self::FIELD_HTTP_HEADER_CONTENT_RANGE);
        $totalItems = null;
        if (preg_match(self::REGEXP_PATTERN_GET_COUNT_ITEM, $contentRange, $matches)) {
            $totalItems = (int)$matches[1];
        }

        if (is_null($totalItems)) {
            throw new \Rest\RestException(500, self::EXCEPTION_UNABLE_CALCULATE_RESOURCES);
        }

        return $totalItems;
    }

    /**
     * @param int $accountId
     * @return \APS\ResourceBase[]
     * @throws Exception
     */
    public function getResellersByAccountId($accountId)
    {
        $rql = RqlBuilder::createInstance()
            ->implementing(\APS\TypeLibrary::getTypeByServiceId('tenant'))
            ->eq('accountId', $accountId)
            ->eq('type', tenant::TYPE_RESELLER)
            ->getQuery();
        return $this->_apsController->getResources($rql);
    }


    /**
     * @param $groupId
     * @param null $datacenterId
     * @return array|null
     */
    public function getTenantsByGroupIdAndDatacenterId($groupId, $datacenterId = null)
    {
        $rqlBuilder = RqlBuilder::createInstance()
            ->implementing(\APS\TypeLibrary::getTypeByServiceId('tenant'))
            ->eq('mspGroupId', $groupId);

        if (!is_null($datacenterId)) {
            $rqlBuilder->eq('datacenter.id', $datacenterId);
        }
        $rql = $rqlBuilder->limit(0, 1)->getQuery();
        $tenants = $this->_apsController->getResources($rql);

        return $tenants;
    }

    /**
     * @param \APS\ResourceBase $account
     * @return \APS\ResourceBase[]
     */
    public function getChildAccounts($account)
    {
        $rql = RqlBuilder::createInstance()
            ->implementing(self::APS_TYPES_PA_ACCOUNT_1_0)
            ->linkedWith($account->aps->id)
            ->getQuery();
        return $this->_apsController->getResources($rql);
    }

    /**
     * @param \APS\ResourceBase $account
     * @return \APS\ResourceBase[]
     */
    public function getAccountSubscriptions($account)
    {
        $rql = RqlBuilder::createInstance()
            ->implementing(self::APS_TYPES_PA_SUBSCRIPTION_1_0)
            ->linkedWith($account->aps->id)
            ->getQuery();
        return $this->_apsController->getResources($rql);
    }

    /**
     * Load list of tenant's staff members info from PA by APS controller, returns first
     * @return object|boolean
     * @throws Exception
     */
    public function getFirstStaffMemberInfo($tenant)
    {
        $staffMembers = $this->getStaffMembers($tenant, $tenant->account, 1);

        if (is_array($staffMembers) && count($staffMembers)) {
            return reset($staffMembers);
        } else {
            throw new Exception(self::EXCEPTION_NO_STAFF_MEMBERS_EXCEPTION);
        }
    }
}
