<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_MspApiClient_Request_Struct_CreateGroup
    implements Service_MspApiClient_Request_Struct_CreateGroupInterface
{
    const FIELD_NAME = "name";
    const FIELD_KIND = "kind";
    const FIELD_PRIVILEGES = "privileges";
    const FIELD_CONTACT = "contact";
    const FIELD_PRICING = "pricing";
    const FIELD_STORAGE = "storage";
    const FIELD_CUSTOMER_ID = "customer_id";
    const FIELD_LANGUAGE = "language";

    /** @var string */
    private $_name = "";
    /** @var int */
    private $_kind = self::EUC_GROUP_KIND_ID;
    /** @var  Service_MspApiClient_Request_Struct_GroupPrivileges */
    private $_privileges;
    /** @var  Service_MspApiClient_Request_Struct_GroupContact */
    private $_contact;
    /** @var  Service_MspApiClient_Request_Struct_GroupPricing */
    private $_pricing;
    /** @var  int */
    private $_storage;
    /** @var  string */
    private $_customer_id;
    /** @var  string */
    private $_language;

    /**
     * @param Service_MspApiClient_Request_Struct_GroupPrivileges $privileges
     * @param Service_MspApiClient_Request_Struct_GroupContact $contact
     * @param Service_MspApiClient_Request_Struct_GroupPricing $pricing
     */
    public function __construct(Service_MspApiClient_Request_Struct_GroupPrivileges $privileges = NULL,
                                Service_MspApiClient_Request_Struct_GroupContact $contact = NULL,
                                Service_MspApiClient_Request_Struct_GroupPricing $pricing = NULL)
    {
        $this->_privileges = is_null($privileges)
            ? new Service_MspApiClient_Request_Struct_GroupPrivileges()
            : $privileges;
        $this->_contact = is_null($contact) ? new Service_MspApiClient_Request_Struct_GroupContact() : $contact;
        $this->_pricing = is_null($pricing) ? new Service_MspApiClient_Request_Struct_GroupPricing() : $pricing;
    }

    /**
     * @return array
     */
    public function toArray()
    {
        $data = array();
        if (!is_null($this->getName())) {
            $data[self::FIELD_NAME] = $this->getName();
        }
        if (!is_null($this->getKind())) {
            $data[self::FIELD_KIND] = $this->getKind();
        }
        if (!is_null($this->getStorage())) {
            $data[self::FIELD_STORAGE] = $this->getStorage();
        }
        if (!is_null($this->getCustomerId())) {
            $data[self::FIELD_CUSTOMER_ID] = $this->getCustomerId();
        }
        if (!is_null($this->getLanguage())) {
            $data[self::FIELD_LANGUAGE] = $this->getLanguage();
        }
        if ($this->getPrivileges()->isExistsUpdateFields()) {
            $data[self::FIELD_PRIVILEGES] = $this->getPrivileges()->toArray();
        }
        if ($this->getContact()->isExistsUpdateFields()) {
            $data[self::FIELD_CONTACT] = $this->getContact()->toArray();
        }
        if ($this->getPricing()->isExistsUpdateFields()) {
            $data[self::FIELD_PRICING] = $this->getPricing()->toArray();
        }

        return $data;
    }

    /**
     * @return Service_MspApiClient_Request_Struct_GroupContact
     */
    public function getContact()
    {
        return $this->_contact;
    }

    /**
     * @param int $kind
     */
    public function setKind($kind)
    {
        $this->_kind = $kind;
    }

    /**
     * @return int
     */
    public function getKind()
    {
        return $this->_kind;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->_name = $name;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->_name;
    }

    /**
     * @return Service_MspApiClient_Request_Struct_GroupPrivileges
     */
    public function getPrivileges()
    {
        return $this->_privileges;
    }

    /**
     * @return Service_MspApiClient_Request_Struct_GroupPricing
     */
    public function getPricing()
    {
        return $this->_pricing;
    }

    /**
     * @param int $storage
     */
    public function setStorage($storage)
    {
        $this->_storage = $storage;
    }

    /**
     * @return int
     */
    public function getStorage()
    {
        return $this->_storage;
    }

    /**
     * @return string
     */
    public function getCustomerId()
    {
        return $this->_customer_id;
    }

    /**
     * @param string $customerId
     */
    public function setCustomerId($customerId)
    {
        $this->_customer_id = $customerId;
    }

    /**
     * @return string
     */
    public function getLanguage()
    {
        return $this->_language;
    }

    /**
     * @param string $language
     */
    public function setLanguage($language)
    {
        $this->_language = $language;
    }
}
