<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_MspApiClient_Request_Struct_UpdateGroup
    extends Service_BaseClient_Request_Struct_Updatable
    implements Service_MspApiClient_Request_Struct_UpdateGroupInterface
{
    const FIELD_STATUS = 'status';
    const FIELD_NAME = 'name';
    const FIELD_CUSTOMER_ID = 'customer_id';
    const FIELD_CONTACT = 'contact';
    const FIELD_PRIVILEGES = 'privileges';
    const FIELD_PRICING = 'pricing';
    const FIELD_PARENT_ID = 'parent_id';
    const FIELD_INTERNAL_TAG = 'internal_tag';
    const DISABLED_GROUP_STATUS = 0;

    /** @var string  */
    private $_name;
    /** @var string */
    private $_customerId;
    /** @var int */
    private $_status;
    /** @var Service_MspApiClient_Request_Struct_UpdateGroupContact */
    private $_contact;
    /** @var Service_MspApiClient_Request_Struct_UpdateGroupPrivileges */
    private $_privileges;
    /** @var Service_MspApiClient_Request_Struct_UpdateGroupPricing */
    private $_pricing;
    /** @var  string */
    private $_parentId;
    /** @var  string */
    private $_internalTag;

    /**
     * @param array $updateMap
     * @param Service_MspApiClient_Request_Struct_UpdateGroupPrivileges $privileges
     * @param Service_MspApiClient_Request_Struct_UpdateGroupContact $contacts
     * @param Service_MspApiClient_Request_Struct_UpdateGroupPricing $pricing
     */
    public function __construct(array $updateMap = array(),
                                Service_MspApiClient_Request_Struct_UpdateGroupPrivileges $privileges = NULL,
                                Service_MspApiClient_Request_Struct_UpdateGroupContact $contacts = NULL,
                                Service_MspApiClient_Request_Struct_UpdateGroupPricing $pricing = NULL)
    {
        parent::__construct($updateMap);

        $this->_privileges = is_null($privileges)
            ? new Service_MspApiClient_Request_Struct_UpdateGroupPrivileges()
            : $privileges;
        $this->_contact = is_null($contacts) ? new Service_MspApiClient_Request_Struct_UpdateGroupContact() : $contacts;
        $this->_pricing = is_null($pricing) ? new Service_MspApiClient_Request_Struct_UpdateGroupPricing() : $pricing;
    }

    /**
     * @return array
     */
    public function toArray()
    {
        $result = array();
        if ($this->_isFieldChanged(self::FIELD_STATUS))
        {
            $result[self::FIELD_STATUS] = $this->getStatus();
        }
        if ($this->_isFieldChanged(self::FIELD_NAME))
        {
            $result[self::FIELD_NAME] = $this->getName();
        }
        if ($this->_isFieldChanged(self::FIELD_CUSTOMER_ID))
        {
            $result[self::FIELD_CUSTOMER_ID] = $this->getCustomerId();
        }

        if ($this->getContact()->isExistsUpdateFields())
        {
            $result[self::FIELD_CONTACT] = $this->getContact()->toArray();
        }

        if ($this->getPrivileges()->isExistsUpdateFields())
        {
            $result[self::FIELD_PRIVILEGES] = $this->getPrivileges()->toArray();
        }

        if ($this->getPricing()->isExistsUpdateFields())
        {
            $result[self::FIELD_PRICING] = $this->getPricing()->toArray();
        }

        if ($this->_isFieldChanged(self::FIELD_PARENT_ID)) {
            $result[self::FIELD_PARENT_ID] = $this->getParentId();
        }

        if ($this->_isFieldChanged(self::FIELD_INTERNAL_TAG)) {
            $result[self::FIELD_INTERNAL_TAG] = $this->getInternalTag();
        }
        return $result;
    }

    /**
     * @param int $status
     */
    public function setStatus($status)
    {
        $this->_markFieldAsUpdated(self::FIELD_STATUS);
        $this->_status = $status;
    }

    /**
     * @return int
     */
    public function getStatus()
    {
        return $this->_status;
    }

    /**
     * @return Service_MspApiClient_Request_Struct_UpdateGroupContact
     */
    public function getContact()
    {
        return $this->_contact;
    }

    /**
     * @return Service_MspApiClient_Request_Struct_UpdateGroupPrivileges
     */
    public function getPrivileges()
    {
        return $this->_privileges;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->_markFieldAsUpdated(self::FIELD_NAME);
        $this->_name = $name;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->_name;
    }

    /**
     * @param string $customerId
     */
    public function setCustomerId($customerId)
    {
        $this->_markFieldAsUpdated(self::FIELD_CUSTOMER_ID);
        $this->_customerId = $customerId;
    }

    /**
     * @return string
     */
    public function getCustomerId()
    {
        return $this->_customerId;
    }

    /**
     * @return Service_MspApiClient_Request_Struct_UpdateGroupPricing
     */
    public function getPricing()
    {
        return $this->_pricing;
    }

    /**
     * @return string
     */
    public function getParentId()
    {
        return $this->_parentId;
    }

    /**
     * @param string $parentId
     */
    public function setParentId($parentId)
    {
        $this->_markFieldAsUpdated(self::FIELD_PARENT_ID);
        $this->_parentId = $parentId;
    }

    /**
     * @return string
     */
    public function getInternalTag()
    {
        return $this->_internalTag;
    }

    /**
     * @param string $internalTag
     */
    public function setInternalTag($internalTag)
    {
        $this->_markFieldAsUpdated(self::FIELD_INTERNAL_TAG);
        $this->_internalTag = $internalTag;
    }
}
