<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Service_OpenApi_HttpTransport implements Service_OpenApi_IHttpTransport
{
    const HTTPS = "https";
    const URL_LEN_FOR_COMPARE = 5;
    const HTTP_CODE = 'httpCode';
    const RESPONSE = 'response';
    const CONNECTION_ERROR = 'Connection error: {%s}\nWas trying to connect to {%s}\nhttp Code: {%s}, {%s}';
    const CONTENT_TYPE_TEXT_XML = "Content-Type: text/xml";
    const TIMEOUT = 300;
    const CONNECTION_TIMEOUT = 10;
    const VERIFY_HOST = FALSE;
    const VERIFY_PEER = FALSE;
    const RETURN_TRANSFER = TRUE;
    const HTTP_HEADER_CONNECTION_CLOSE = 'Connection: close';

    /** @var CurlInterface  */
    private $_curl;

    /**
     * @param CurlInterface $curl
     */
    public function __construct(CurlInterface $curl = NULL)
    {
        $this->_curl = is_null($curl) ? new Curl() : $curl;
    }

    /**
     * @param string $method
     * @param string $uri
     * @param string $data
     * @return Service_OpenApi_Response_RawInterface
     * @throws Service_OpenApi_HttpTransport_ConnectionException
     */
    public function sendRequest($method, $uri, $data = NULL)
    {
        $this->_curl->setOption(CURLOPT_URL, $uri);
        $this->_curl->setOption(CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        $this->_curl->setOption(CURLOPT_CONNECTTIMEOUT, self::CONNECTION_TIMEOUT);
        $this->_curl->setOption(CURLOPT_TIMEOUT, self::TIMEOUT);
        $this->_curl->setOption(CURLOPT_RETURNTRANSFER, self::RETURN_TRANSFER);
        $this->_curl->setOption(CURLOPT_HTTPHEADER, array(
            self::CONTENT_TYPE_TEXT_XML,
            self::HTTP_HEADER_CONNECTION_CLOSE
        ));
        $this->_curl->setOption(CURLOPT_CUSTOMREQUEST, $method);
        $this->_curl->setOption(CURLOPT_POSTFIELDS, $data);
        $this->_curl->setOption(CURLOPT_SSL_VERIFYHOST, self::VERIFY_HOST);
        $this->_curl->setOption(CURLOPT_SSL_VERIFYPEER, self::VERIFY_PEER);

        $response = $this->_curl->execute();
        $httpCode = $this->_curl->getInfo(CURLINFO_HTTP_CODE);

        $error = $this->_curl->getError();
        if ($response === false)
        {
            throw new Service_OpenApi_HttpTransport_ConnectionException(
                sprintf(self::CONNECTION_ERROR,
                    $error,
                    $uri,
                    $httpCode,
                    $this->_curl->getErrorNo()
                )
            );
        }

        return new Service_OpenApi_Response_Raw($httpCode, $response);
    }
}