<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

interface Service_PbaApi_ApiInterface
{
    const ORDER_STATUS_NW = 'NW';
    const ORDER_STATUS_OP = 'OP';
    const ORDER_STATUS_CP = 'CP';
    const ORDER_STATUS_PF = 'PF';
    const ORDER_STATUS_PD = 'PD';

    public function startTransaction();

    public function commitTransaction();

    public function rollbackTransaction();

    /**
     * @param int $accountId
     * @param int $discountId
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function addAccountInDiscount($accountId, $discountId);

    /**
     * @param int $accountId
     * @param int $discountId
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function moveToArchiveAccountInDiscount($accountId, $discountId);

    /**
     * @param int $subscriptionId
     * @param string $parameterId
     * @return mixed
     */
    public function getSubscriptionParameterValue($subscriptionId, $parameterId);

    /**
     * @param int $objectType
     * @param int $objectId
     * @return Service_PbaApi_Api_Response_Struct_ObjectAttributesListItemInterface[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getObjectAttributesList($objectType, $objectId);

    /**
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_AttributeTypeListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getAttributesTypeList($sortNo = 1);

    /**
     * Method returns the list of published service plans of specified plan category.
     *
     * @param int $accountID account ID of plan category owner
     * @param string $stType name of service gate service plans are provisioned through: PEMGATE, DOMAINGATE, DUMMYGATE, etc.
     * @param int $planCategoryID plan category ID
     * @param int $sortNo parameter defines how output data is sorted:
     *    "1" - the output is sorted by plan ID in ascending order,
     *    "2" - by plan name in ascending order, etc. Negative value makes output sorted in descending order.
     * @return Service_PbaApi_Api_Response_Struct_AvailablePlanListItemInterface[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getAvailablePlanList($accountID, $stType, $planCategoryID, $sortNo = 1);

    /**
     * @param int $orderId
     * @throws Service_PbaApi_Api_Exception
     */
    public function openOrder($orderId);

    /**
     * @param int $planID
     * @return Service_PbaApi_Api_Response_Struct_PlanInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlan($planID);

    /**
     * @param int $planId
     * @return Service_PbaApi_Api_Response_Struct_PlanDetails
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanDetails($planId);

    /**
     * @param $categoryId
     * @return Service_PbaApi_Api_Response_Struct_PlanCategoryDetails
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanCategoryDetails($categoryId);

    /**
     * Method returns the list of plan categories for specified vendor account.
     *
     * @param int $accountId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_PlanCategoryListItemInterface[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanCategoryList($accountId, $sortNo = 1);

    /**
     * @param int $accountId
     * @return Service_PbaApi_Api_Response_Struct_PlanInterface[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getServicePlans($accountId);


    /**
     * @param int $accountId
     * @param int $planId
     * @return Service_PbaApi_Api_Response_Struct_PlanInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function getServicePlan($accountId, $planId);

    /**
     * @param int $planId
     * @param int $serviceTemplateID
     * @param Service_PbaApi_Api_Request_Struct_ServiceTemplateParameter[] $parameters
     * @return Service_PbaApi_Api_Response_Struct_GetAndValidatePlanParams[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getAndValidatePlanParameters($planId, $serviceTemplateID, array $parameters = array());

    /**
     * @param integer $subscriptionId
     * @param integer $status
     * @param integer $serviceStatus
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionDetailsInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function updateSubscriptionStatus($subscriptionId, $status, $serviceStatus);

    /**
     * @param integer $subscriptionId
     * @param int $expirationDate
     * @param string $reason
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionDetailsInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function extendTrialPeriod($subscriptionId, $expirationDate, $reason);

    /**
     * @param integer $subscriptionId
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionDetailsInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionDetails($subscriptionId);

    /**
     * @param integer $subscriptionId
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionDetailsEx
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionDetailsEx($subscriptionId);

    /**
     * @param $orderId
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionsListByOrder($orderId);

    /**
     * @param int $subscriptionId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionResourcesList[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionResourcesList($subscriptionId, $sortNo = 0);

    /**
     * @param int $subscriptionId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_GetSubscriptionSalesOrdersListInterface[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionSalesOrdersList($subscriptionId, $sortNo = 0);

    /**
     * @param int $subscriptionId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionAvailablePlanListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getSubscriptionAvailablePlanList($subscriptionId, $sortNo = 0);

    /**
     * @param $subscriptionId
     * @param $usePayTool
     * @param $useDefaultPayTool
     * @param $userId
     * @param $fromIp
     * @param Service_PbaApi_Api_Request_Struct_ResourceUpgradeOrder[] $resourceUpgradeOrders
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function placeResourceUpgradeOrder($subscriptionId, $usePayTool, $useDefaultPayTool, $userId, $fromIp, $resourceUpgradeOrders);

    /**
     * @param int $orderId
     * @return Service_PbaApi_Api_Response_Struct_GetOrder
     * @throws Service_PbaApi_Api_Exception
     */
    public function getOrder($orderId);

    /**
     * @param int $orderId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_GetOrderDetails[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getOrderDetails($orderId, $sortNo = 0);

    /**
     * @param int $orderId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_GetOrderDetails[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getOrderProvisioningItemsList($orderId, $sortNo = 0);

    /**
     * @param int $orderId
     * @param string $status
     * @param string $signature
     * @return Service_PbaApi_Api_Response_Struct_OrderProvisioningItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function changeOrderStatus($orderId, $status, $signature);

    /**
     * @param $attributeId
     * @param $attributeValue
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_GetAccountListByCustomAttribute[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getAccountListByCustomAttribute($attributeId, $attributeValue, $sortNo = 1);

    /**
     * @param integer $subscriptionId
     * @param integer $newExpData
     * @param string $comment
     * @return Service_PbaApi_Api_Response_Struct_SubscriptionDetailsInterface
     * @throws Service_PbaApi_Api_Exception
     */
    public function changeExpiration($subscriptionId, $newExpData, $comment);

    /**
     * @param $vendorAccountID
     * @return Service_PbaApi_Api_Response_Struct_GetVendorCurrency
     * @throws Service_PbaApi_Api_Exception
     */
    public function getVendorCurrency($vendorAccountID);

    /**
     * @param $planId
     * @param int $sort
     * @return Service_PbaApi_Api_Response_Struct_PlanPeriodListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanPeriodList($planId, $sort = 1);

    /**
     * @param $planId
     * @param int $sort
     * @return Service_PbaApi_Api_Response_Struct_PlanRateListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanRateList($planId, $sort = 1);

    /**
     * @param $planPeriodId
     * @param int $sort
     * @return Service_PbaApi_Api_Response_Struct_PlanRateListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanRateListForPeriod($planPeriodId, $sort = 1);

    /**
    * @param $planId
    * @param int $sortNo
    * @return Service_PbaApi_Api_Response_Struct_PlanRate[]
    * @throws Service_PbaApi_Api_Exception
    */
    public function getPlanRates($planId, $sortNo = 1);

    /**
     * @param $vendorAccountId
     * @param array $parameters
     * @param string|null $language
     * @return mixed
     */
    public function placeOrderAndAuthorize($vendorAccountId, $parameters, $language = null);

    /**
     * @param int $subscriptionID
     * @param int $planID
     * @param int $periodID
     * @param int $upgradeStartType
     * @param int $applyPromo
     * @return Service_PbaApi_Api_Response_Struct_PlacePlanPeriodSwitchOrder
     * @throws Service_PbaApi_Api_Exception
     */
    public function placePlanPeriodSwitchOrder($subscriptionID, $planID, $periodID, $upgradeStartType, $applyPromo);

    /**
     * @param int $subscriptionId ID of subscription to be cancelled.
     * @param int $cancelType cancellation type:
     * 10 – customer is refunded. Corresponds to the behavior of subscription, for which all checkboxes of
     * Subscription Cancellation Options (Control Panel) are selected. The refund, overuse and cancellation fees are taken.
     * 20 – customer account is credited (credit memo is created). The refund, overuse and cancellation fees are taken.
     * 30 – customer is not refunded / credited. Corresponds to the behavior of subscription, for which all checkboxes of
     * Subscription Cancellation Options (Control Panel) are deselected. No refund, overuse or cancellation fees are taken.
     * @param int $reasonId ID of system reason code to be stated as reason of subscription cancellation.
     * @param string $description optional comment on the reason subscription is cancelled.
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function placeSubscriptionCancellationOrder($subscriptionId, $cancelType, $reasonId, $description = '');

    /**
     * @param Service_PbaApi_Api_Request_Struct_OrderAttributes $orderAttributes
     * @param Service_PbaApi_Api_Request_Struct_OrderDetailAttributes[] $orderDetails
     * @param Service_PbaApi_Api_Request_Struct_TaxDetailAttributes[] $taxDetails
     * @return int Order ID
     * @throws Service_PbaApi_Api_Exception
     */
    public function createManualOrder(Service_PbaApi_Api_Request_Struct_OrderAttributes $orderAttributes, array $orderDetails, array $taxDetails = array());

    /**
     * @param $attributeType
     * @param $objId
     * @param Service_PbaApi_Api_Request_Struct_UpdateObjAttrItem[] $attributeToUpdate
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function updateObjAttrList($attributeType, $objId, array $attributeToUpdate);

    /**
     * Don't use if not sure
     * Returns some information about the resource
     * @param $resourceId
     * @return Service_PbaApi_Api_Response_Struct_GetResource
     * @throws Service_PbaApi_Api_Exception
     */
    public function getResource($resourceId);

    /**
     * Don't use if not sure
     * Returns some information about the included resource
     * @param $resourceId
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function getIncludedResources($resourceId);

    /**
     * @param $accountId
     * @return Service_PbaApi_Api_Response_Struct_Account
     * @throws Service_PbaApi_Api_Exception
     */
    public function getAccount($accountId);

    /**
     * @param int $discountId
     * @return Service_PbaApi_Api_Response_Struct_Discount
     * @throws Service_PbaApi_Api_Exception
     */
    public function getDiscountById($discountId);

    /**
     * @param integer $subscriptionId
     * @param string $parameterId
     * @param string $parameterValue
     * @return mixed
     * @throws Service_PbaApi_Api_Exception
     */
    public function updateSubscriptionParameterValue($subscriptionId, $parameterId, $parameterValue);

    /**
     * @param integer $splitDate
     * @param integer $vendorId
     * @return Service_PbaApi_Api_Response_Struct_Status
     * @throws Service_PbaApi_Api_Exception
     */
    public function splitDeliverLinesForVendorByDate($splitDate, $vendorId);

    /**
     * @param int $fromDate
     * @param int $toDate
     * @param int $vendorId
     * @return Service_PbaApi_Api_Response_Struct_Status
     * @throws Service_PbaApi_Api_Exception
     */
    public function mergeDeliverLinesForPeriodByVendor($fromDate, $toDate, $vendorId);

    /**
     * @param int $fromDate
     * @param int $toDate
     * @param int $vendorId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_DeliverLine[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getDeliverLinesForPeriodByVendorList($fromDate, $toDate, $vendorId, $sortNo = 0);

    /**
     * @param int $planId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_PlanHistoryListItem[]
     */
    public function getPlanHistoryList($planId, $sortNo = 0);

    /**
     * @param int $planId
     * @param int $userId
     * @param int $date
     * @return mixed
     */
    public function getPlanFromArchive($planId, $userId, $date);

    /**
     * @param int $planId
     * @return Service_PbaApi_Api_Response_Struct_PlanPublicationStatus
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanPublicationStatusForUpdate($planId);

    /**
     * @param int $planId
     * @param int $published
     * @param int $showPriority
     * @param int $defaultPlanPeriodId
     * @return Service_PbaApi_Api_Response_Struct_Status
     * @throws Service_PbaApi_Api_Exception
     */
    public function updatePlanPublicationStatus($planId, $published, $showPriority, $defaultPlanPeriodId);

    /**
     * @param int $discountId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_DiscountedResourceListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getDiscountedResourceList($discountId, $sortNo = 0);

    /**
     * @param int $discountId
     * @param int $sortNo
     * @return Service_PbaApi_Api_Response_Struct_DiscountedPlanListItem[]
     * @throws Service_PbaApi_Api_Exception
     */
    public function getDiscountedPlanList($discountId, $sortNo = 0);

    /**
     * @param $detailId
     * @return Service_PbaApi_Api_Response_Struct_OrderDetail
     * @throws Service_PbaApi_Api_Exception
     */
    public function getOrderDetail($detailId);

    /**
     * @param $periodId
     * @return Service_PbaApi_Api_Response_Struct_PlanPeriod
     * @throws Service_PbaApi_Api_Exception
     */
    public function getPlanPeriod($periodId);
}
