<?php

namespace Acronis\App\Services\NotificationManager;

use Acronis\App\CloudApi\Api;
use Arr;

/**
 * Class NotificationManager
 * Basic idea is here managing user subscriptiable notifications on cloud
 * @package Acronis\App\Base
 */
class NotificationManager
{
    const SUBSCRIPTIABLE_NOTIFICATIONS = [
        'anti_malware' => 'com.acronis.protection.amp.notifications'
    ];

    /**
     * @var Api
     */
    private $api;

    /**
     * @var array
     */
    public static $instances = [];

    /**
     * NotificationManager constructor.
     * @param Api $api
     */
    public function __construct(Api $api)
    {
        $this->api = $api;
    }

    /**
     * @param $userId
     * @param $notifications
     * @throws \Acronis\Cloud\Client\ApiException
     */
    public function updateUserNotifications($userId, $notifications)
    {
        $enabledNotifications = [];
        $disabledNotifications = [];
        $subscribedNotificationsHash = [];

        $subscribedNotifications = $this->getSubscribedUserNotifications($userId);
        foreach ($subscribedNotifications as $subscribedNotification) {
            $subscribedNotificationsHash[$subscribedNotification->name] = $subscribedNotification;
        }

        foreach (static::SUBSCRIPTIABLE_NOTIFICATIONS as $key => $notification) {
            if (in_array($key, $notifications) && $subscribedNotificationsHash[$notification]->subscribed === false) {
                $enabledNotifications[] = $notification;
            }
            if (!in_array($key, $notifications) && $subscribedNotificationsHash[$notification]->subscribed === true) {
                $disabledNotifications[] = $notification;
            }
        }

        if ($enabledNotifications) {
            $this->api->updateUserSubscribedNotifications($userId, $enabledNotifications, true);
        }

        if ($disabledNotifications) {
            $this->api->updateUserSubscribedNotifications($userId, $disabledNotifications, false);
        }
    }

    /**
     * @param $userId
     * @return \Acronis\Cloud\Client\HttpResponseInterface|mixed
     * @throws \Acronis\Cloud\Client\ApiException
     * @throws \Acronis\Cloud\Client\HttpException
     */
    public function getSubscribedUserNotifications($userId)
    {
        return $this->api->getUserSubscribedNotifications($userId);
    }

    /**
     * @param $notifications
     * @return array
     */
    public static function getSubscriptiableNotifications($notifications)
    {
        return array_values(array_filter($notifications ?? [], function ($notification) {
            return in_array($notification, array_keys(static::SUBSCRIPTIABLE_NOTIFICATIONS));
        }));
    }

    /**
     * @param $notifications
     * @return array
     */
    public static function getUpdatableNotifications($notifications)
    {
        return array_values(array_filter($notifications, function ($notification) {
            return !in_array($notification, array_keys(static::SUBSCRIPTIABLE_NOTIFICATIONS));
        }));
    }

    /**
     * @param $subscriptions
     * @return array
     */
    public static function convertEnabledSubscriptionsToNotifications($subscriptions)
    {
        $enabledSubscriptions = array_filter($subscriptions, function ($subscription) {
            return $subscription->subscribed && in_array($subscription->name, self::SUBSCRIPTIABLE_NOTIFICATIONS);
        });

        return array_map(function ($subscription) {
            return Arr::get(array_flip(self::SUBSCRIPTIABLE_NOTIFICATIONS), $subscription->name);
        }, $enabledSubscriptions);
    }
}