<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class EndpointHostIpFetcher
{
    const EXCEPTION_ENDPOINT_HOST_NOT_FOUNT = 'Endpoint host not found';

    /**
     * @var \APS\ControllerProxy
     */
    private $apsc = null;

    public function __construct(\APS\ControllerProxy $apsc)
    {
        $this->apsc = $apsc;
    }

    /**
     * @return mixed|null
     * Returns the IP address of the endpoint involved in operation.
     * Falls back to returning IP address of node executing the method.
     *
     * @throws Exception
     */
    public function fetchApsEndpointIpAddressOrFallback()
    {
        $services = json_decode($this->apsc->getApplicationServices());
        $host = null;
        if (isset($services->aps) && isset($services->aps->endpoint)) {
            $host = parse_url($services->aps->endpoint, PHP_URL_HOST);
        }
        if ((null === $host) || (false === $host)) {
            $retVal = null;
            $outputStub = array();
            $listOfHostIps = exec('hostname -I', $outputStub, $retVal);
            if ((0 === $retVal) && (0 < strlen($listOfHostIps))) {
                list($host) = explode(' ', $listOfHostIps);
            }
        }

        if (is_null($host)) {
            throw new Exception(self::EXCEPTION_ENDPOINT_HOST_NOT_FOUNT);
        }

        return $host;
    }
}